#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>

#define PAYLOAD_LLC  18
#define IP4_HDR      20
#define IP6_HDR      40
#define UDP_HDR      8

int usage();

int main(int argc, char *argv[]) {
	int sel_opt, is_verbose, is_ipv6, is_raw, is_ip, is_kbps;
	int mida_paquet, mida_paquet_ip, bit_paquet, cela_paquet;
        int bw_eth, bps_ip, rate_;

	// Initialitzations
	is_ipv6 = 0; is_verbose = 0, is_raw = 0, is_ip = 0, is_kbps = 0;
	rate_ = -1;

	// Main option parsing loop
	opterr = 0;
	while(1) {
		sel_opt = getopt(argc, argv, "vhrikR:6");
		if (sel_opt == -1)
			break;
		switch (sel_opt) {
		case 'v':
			is_verbose = 1;
		break;
		case 'h':
			usage();
			exit(0);
		break;
		case 'r':
			is_raw = 1;
		break;
		case 'i':
			is_ip = 1;
		break;
		case 'k':
			is_kbps = 1;
		break;
		case 'R':
			rate_ = atoi(optarg);
		break;
		case '6':
			is_ipv6 = 1;
		break;
		default:
			usage();
			exit(-2);
		break;
		}
	}
	if (rate_ <= 0) {
		fprintf(stderr,"Rate is a mandatory argument and has to be > 0\n");
		usage();
		exit(-1);
	}
	if (!argv[optind]) {
		fprintf(stderr,"Packet size is mandatory\n");
		usage();
		exit(-1);
	}
	if (is_ipv6) {
		if (is_raw)
			mida_paquet_ip =  atoi(argv[optind]);
		else
			mida_paquet_ip =  atoi(argv[optind]) + IP6_HDR + UDP_HDR;
	} else {
		if (is_raw)
			mida_paquet_ip = atoi(argv[optind]);
		else
			mida_paquet_ip = atoi(argv[optind]) + IP4_HDR + UDP_HDR;
	}

	if (mida_paquet_ip <= 20) {
		fprintf(stderr,"The UDP payload has to be bigger than 20\n");
		return -1;
	}

	mida_paquet = mida_paquet_ip + PAYLOAD_LLC;
	bit_paquet = mida_paquet * 8;
	bw_eth = rate_ * bit_paquet;
	bps_ip = rate_ * mida_paquet_ip * 8;
	if (is_verbose) {
		fprintf(stdout,"Mida paquet IP amb capalera: %d bytes\n",mida_paquet_ip);
		fprintf(stdout,"BW(ETH) = %d%s\n",is_kbps ? rate_/1024 : rate_, is_kbps ? "Kbps" : "bps");
		fprintf(stdout,"BW(IP)  = %d%s\n",is_kbps ? bps_ip/1024 : bps_ip, is_kbps ? "Kbps" : "bps");
	} else if (is_ip) {
		fprintf(stdout,"%d",is_kbps ? bps_ip/1024 : bps_ip);
	} else {
		fprintf(stdout,"%d",is_kbps ? bw_eth/1024 : bw_eth);
	}
	return 0;
}

int usage() {
	fprintf(stdout,"usage: ./bw [-v] [-6] [-l] [-r] [-e] [-k] -R pkt/sec <mida_paquet>\n");
	fprintf(stdout,"\tUtilitat per comptar l'amplada de banda que ocupa un rate realment a nivell Ethernet\n");
	fprintf(stdout,"  -v: Verbose, mostra informaci extra per pantalla\n");
	fprintf(stdout,"  -i: BPS are at IP level\n");
	fprintf(stdout,"  -6: Suposem que treballem amb paquets IPv6\n");
	fprintf(stdout,"  -r: Raw Mode, es suposa que la mida del paquet inclou fins capaleres IP [inclusive]\n");
	fprintf(stdout,"  -k: Les unitats de sortida de l'amplada de banda seran amb Kbps\n");
	return 0;
}

