/* ###############################################
   # Copyright 2002       Roberto Borgione       #
   #                      Manuel González        #
   #                                             #
   # distr.c - Distribution parser and calc.     #
   #       v2.0 (03-2002)                        #
   ###############################################
 
   This file is part of NetMeter.

   NetMeter is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   NetMeter is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with NetMeter; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// Tratamiento dinamico de las secuencias mediante un AVL	
// Prueba efectuada con éxito para ficheros .dly extensos (1.000.000) secuencias
// Errores de violacion de segmento, buffer owerflow, cierre de descriptores  y acceso a variables no inicializadas corregidos
// Estructuración y limpieza del codigo. Comentarios.
// Generación de ficheros temporales en el directorio /tmp
// Nombre unico para los ficheros generados y por tanto permitida la ejecución de varias instancias en la misma máquina
// Suprimir ficheros temporales cuando ya no son necesarios
// Añadida función adicional para tratamiento de errores.
// Tratamiento de los paquetes perdidos para las opciones que lo precisen; queda de la siguiente manera:
//    -owd: ordenación de paquetes y escritura de 0.0 en pérdidas.
//    -owd distribution: ordenación de paquetes (no se contemplan las perdidas, es una distribución)
//    -ipdv: ordenación de paquetes. Se contemplan las pérdidas para suprimir las secuencias a las que afecten las mismas (sec. perdida + anterior) y para
//           escribir el numero de secuencia en cuestión (para posteriormente dejar constancia de los intervalos con pérdidas).
//    -ipdv distribution: ordenación. Se contemplan las pérdidas para suprimir las secuencias a las que afecten las mismas
//    -owd average: ordenación. No se contemplan las perdidas, es una distribución
//
// Añadida estructura para contemplar la opción de parámetros introducidos opcionalmente por fichero
// Eliminar variables globales referentes a parámetros de entrada y accesos aleatorios a argv o argc
// Ejecución de graph2002 desde distr.c (para pasar la identificación unica del fichero graphDAT_X
// 
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// prompt command:  - - 0 1 1 100 1 100 100 1 1 1 - h:/work/data1b.dly 1 h:/work/data1n.dly 1

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <unistd.h>
#include <arbolAVL.h>

//CONSTANTS 

//Relative to ./netmeter/DIRECTORY_OUT
#define DIRECTORY_FILE_OUT "/tmp/"    

#define MAX_graphic_points  1500
#define MAX_LINE_LENGHT  300
#define MAX_INPUT_FILES  25
#define first_file 14
//how many files have to jump
#define TRANSITORY_LINES 30   


  char Ytitle[9][40]={              "% of packets",     "% of packets"," EMPTY"," EMPTY","delay (msec)",    "unkown",                      "% of packets" ,                 "IPDV","unknown"};
  char Xtitle[9][40]={              "delay (msec)",      "ipdv (msec)"," EMPTY"," EMPTY","packet sequence number",   "unknown","distance from average delay (msec)","packet sequence number","unknown"};
  char   task[9][60]={"One-Way delay distribution","IPDV distribution"," EMPTY"," EMPTY","One-Way delay","UPDATE MCL",        "One-Way delay distribution",                  "IPDV","unknown"};
  char   flag[2][40]={"disable","enable"};

//VARS

   float old_gap,old_min;
   float   flow_max[MAX_INPUT_FILES];
   float   flow_min[MAX_INPUT_FILES];
   char  fnames[MAX_INPUT_FILES][300];
   int   nflows[MAX_INPUT_FILES];
   float average[MAX_INPUT_FILES];
   float auxvar,total_delay;
   float gap;
   long  var;
   int   countp,countf,count,countx,county,countk,temp;
   char   hor1[3], min1[3],  sec1[10];
   char   hor2[3], min2[3],  sec2[10],seq[10];
   int   _hor1,   _min1;
   int   _hor2,   _min2;
   float _sec1;
   float _sec2;
   int   SeqNum,max_SeqNum,min_SeqNum;
   char  size[300];
   FILE *fout=NULL;
   FILE *f=NULL;
   char *fout_name;
   char  proc[80];
   float delay,min,max,aux,last,ipdv;
   int   fnum,x;
   float var1,var2;
   float   count_dat[MAX_INPUT_FILES];
   int    last_SeqNum[MAX_INPUT_FILES],pack_size[MAX_INPUT_FILES];
   char   flow_title[30][300];
   char        line3[MAX_LINE_LENGHT];
   char        line2[MAX_LINE_LENGHT];
   char         line[MAX_LINE_LENGHT];
   FILE        *faux[MAX_INPUT_FILES];
   float    vartotal[MAX_INPUT_FILES];
   unsigned int dist[MAX_INPUT_FILES][MAX_graphic_points+1];
   int time_stamp_offset,fixed_flow,read_flow;
   char ipver, flownumber[10];
   float real_step;
   float real_size;
   char minf,maxf; 

   FILE *ft;
   int data1,data2;
   int first;
   char stemp[300];
   int countflow;
   char *ptr;
   char counta,countb,countc;
   FILE *fm;

   char mname[300];
   char xname[300];
   char xline[500], yline[500];

   //STRUCT TO GET ARGUMENTS
   struct distr_parameters {

	//ORDENADOS POR ORDEN DE PARAMETRO: 1,2..
	//min time y max time
	char argv_min[20];
	char argv_max[20];
	//Opcion: IPDV, OWD, OWDD, IPDVD ...
	char opt;	
	//Transitory analisys
	char trans;
	//Fichero PNG
	char out_name[300];	
	//X size
	int graphic_points;
	
	//Pixel Gap
	char esp[10];
	//Text Gap
	char xes[10];
	//Y size
	char ty[10];
	//Fill graphic distribution
	char rfill;
	//Zero reference for axis X
	char zrefx;
	//Use extra layout
	char rback;
	//Enable Pattern Gap (pixel)
	char rpattern[10];

	char  fnames[MAX_INPUT_FILES][300];
	int   nflows[MAX_INPUT_FILES];	

   };


/////////////////////////////////////////////////////////////////////
//   
//FUNCTIONS   	
//
////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////
//OPEN ALL TEMP FILES (timeN_f0..) TO CALCULATE POINTS
////////////////////////////////////////////////////////////////////

void open_all_temp_files( void )
{
   
	// Opens all output file generated
	for(countf=0; countf<fnum; countf++) 
	{
		proc[4]=(int)(countf + '0');
      		printf("abro fichero: %s\n", fout_name);
      		faux[countf]=fopen(fout_name,"r");
   
   	}
}

//////////////////////////////////////////////////////////////////
//DELETE ALL TEMP FILES WHERE THEY ARE'NT NECESARY
//////////////////////////////////////////////////////////////////

void delete_temp(char *identificacion)
{
   	int res;
	char aux_remove[50];
   	for(countf=0; countf<fnum; countf++)  
   	{
     
	      	proc[4]=(int)(countf + '0');
      	      	sprintf(aux_remove,"/tmp/%s", fout_name);
      	      	printf("%s\n", aux_remove);
		res=unlink(aux_remove);
   		if (res == -1) {
			printf("\nError deleting temp files (timeN..)\n");
		}
	}
}

///////////////////////////////////////////////////////////////////
//CLOSE ALL TEMP FILES ONCE WRITTEN
///////////////////////////////////////////////////////////////////

void close_all_temp_files( void )
{
   	for(countf=0;countf<fnum;countf++)fclose(faux[countf]);
}


/////////////////////////////////////////////////////////////////////////////////
//GENERATES COORDS. USING DELAY FILES FOR OWD DISTRIBUTION AND IPDV DISTRIBUTION
/////////////////////////////////////////////////////////////////////////////////


void process_IPDV_ONEWAY (char *identificacion, struct distr_parameters *dparam)           
{
   	max=max+0.0002;
   	min=min-0.0002;
   	char aux[50];	

   	sprintf(aux, "graphDAT%s", identificacion);

   	if(strcmp(dparam->argv_min,"-")) min=(atof(dparam->argv_min))*0.001;
   	if(strcmp(dparam->argv_max,"-")) max=(atof(dparam->argv_max))*0.001;

   	open_all_temp_files();
   	gap=(max-min);
   	real_size=dparam->graphic_points;
   	real_step=gap/real_size;

   	for(countf=0; countf<fnum; countf++) {
      		while(!feof(faux[countf])) {
   	 		fgets(line,256,faux[countf]);
	 		if (feof(faux[countf])) { 
				break;
			}
	 			
			delay=atof(line);
	 		var=(delay-min)/real_step;
         
	  		if ( (var > dparam->graphic_points) || (var < 0) ) {
            			//printf(" fatal RANGE_VALUE error.....!!!\n");
            			//exit(-1);
         		}
         		else {
				dist[countf][var]++;
      			}
		}
   	}
   
	close_all_temp_files();
   	f=fopen(aux,"w");
   
	for (countp=0; countp<=dparam->graphic_points; countp++)
   	{
      		//TIME IN msec
      		fprintf(f,"%.2f ", (min + countp * real_step) * 1000);  // time in msec
      		
		//AXIS-Y in %
		for(countf=0; countf<fnum; countf++)         
      		{
			fprintf(f, "%f ", 100 * dist[countf][countp] / count_dat[countf]);
      		}
      		fprintf(f,"\n");
   	}
   	fclose(f);
   
}

////////////////////////////////////////////////////////////
//GENERATE COORD. USING DELAY FILES FOR OWD 
////////////////////////////////////////////////////////////

void process_HEAD (char *identificacion, struct distr_parameters *dparam)           
{
	int ph_istep, i, ph_iseqnum, ph_max_SeqNum;
	float ph_fdelay[MAX_INPUT_FILES];
	char aux[50];   

	sprintf(aux, "graphDAT%s", identificacion);
   
   	open_all_temp_files();
	f=fopen(aux,"w");
   
	ph_istep = 0;
   	ph_max_SeqNum = last_SeqNum[0];
   
   	for (count=1; count < fnum; count++) {
	   	if (last_SeqNum[count] > ph_max_SeqNum) {
		   	ph_max_SeqNum = last_SeqNum[count];
   		}
	}
   
   	if (dparam->graphic_points < ph_max_SeqNum) {
	   		ph_istep = (ph_max_SeqNum + 1)/ dparam->graphic_points;
   	} 
	else if (dparam->graphic_points > ph_max_SeqNum) {
		   	ph_istep = -1;
   	}
   

	if (ph_istep <= 0) {
		for (count=0; count < ph_max_SeqNum; count++) {
			fprintf(f,"%d ",count);
		   	for(countf = 0; countf < fnum; countf++) {
			   	fgets(line, 256, faux[countf]);
			   	delay=atof(line);
			   	fprintf(f, "%f ", delay * 1000);
		   	}
		   	fprintf(f, "\n");
	   	}
	   	for (count = ph_max_SeqNum; count < dparam->graphic_points; count++) {
		   	fprintf(f, "%d 0.0\n", count);
	   	}
	   	close_all_temp_files();
	   	fclose(f);
		return;
   	}
   
	//Adjust means, graphic smaller than packets
   
	for (i = 0; i < countf; i++){
	   	ph_fdelay[i] = 0;
	}   

	for (count=1; count <= (ph_max_SeqNum + 1); count++) {
		for (countf=0; countf < fnum; countf++) {
		   	fgets(line, 256, faux[countf]);
		   	ph_fdelay[countf] += atof(line);
	   	}
	   	if (!(count % ph_istep)) {
		   	fprintf(f, "%d ", count - ph_istep + 1);
		   	for (countf = 0; countf < fnum; countf++) {
			   	fprintf(f, "%f ", ph_fdelay[countf] * 1000 / ph_istep);
		   	}
		   	for (i = 0; i < countf; i++) {
			   	ph_fdelay[i] = 0;
		  	}
			fprintf(f,"\n");
	   	}
   	}
   
	fprintf(f,"%d ",ph_istep * (dparam->graphic_points + 1) + 1);
   
	for (countf = 0; countf < fnum; countf++) {
	   	fprintf(f,"%f ",ph_fdelay[countf] * 1000 / (ph_max_SeqNum - (ph_istep * (dparam->graphic_points + 1)) + 1));
   	}
   	fprintf(f, "\n");
 	  
   	close_all_temp_files();
	fclose(f);
}

////////////////////////////////////////////////////////////////
//internal of update MCL files
////////////////////////////////////////////////////////////////

void update_file(void)
{
	if (data1==0)
        {
        	fputs("Update Data1:\n",ft);
            	fputs("-[calc] Average Delay (sec):",ft);
            	sprintf(stemp," %f\n", average[countk]);
            	fputs(stemp,ft);
            	fputs("-[calc] Variance (msec^2):",ft);
            	sprintf(stemp," %f\n", vartotal[countk]);
            	fputs(stemp,ft);
            	fputs("-[calc] Standard deviation (msec) :",ft);
            	sprintf(stemp," %f\n", sqrtf(vartotal[countk]) );
            	fputs(stemp,ft);
         }
}

////////////////////////////////////////////////////////////////
//OPTION 5 ....
////////////////////////////////////////////////////////////////

int  process_UPDATE_MCL(void)
{
	open_all_temp_files();
   	for(countf=0; countf<fnum; countf++)
   	{
      		auxvar=0;
      		while(!feof(faux[countf]))
      		{
      	 		//add#num[sqr(x-aver.)]/#num
			fgets(line,256,faux[countf]); 
 	 		if (feof(faux[countf])) {
				 break;
			}
         		
			delay=atof(line);
         
         		// for better precision time->msec
			var1=delay*1000-average[countf]*1000;
         		// simple SQR 
			delay=var1*var1;          
         		auxvar = auxvar + delay;
      		}
      		//time->sec
		auxvar=auxvar/count_dat[countf];
      		vartotal[countf]=auxvar;
      		printf("\n\n Variance  IN = %f\n\n",vartotal[countf]);
   	}
   
	close_all_temp_files();
   
	for(countk=0; countk<fnum; countk++)
   	{
      		strcpy(mname,fnames[countk]);
      		mname[strlen(mname)-3]='m';
      		mname[strlen(mname)-2]='c';
      		mname[strlen(mname)-1]='l';

      		strcpy(xname,fnames[countk]);
      		xname[strlen(xname)-3]='t';
      		xname[strlen(xname)-2]='m';
      		xname[strlen(xname)-1]='p';


      		ft=fopen(xname,"w");
      		fm=fopen(mname,"r");
      
		if (fm==NULL) {
         		fclose(ft);
         		printf("\n MGEN - Distribution generator:\n\n ERROR...!!! MCL file not found.\n");
         		return(-1);
      		}
      		else
      		{
         		data1=0;
         		data2=0;
         		countflow=0;
         
			while(!feof(fm))
         		{
            			fgets(xline,500,fm);
            			if (feof(fm)) {
					  break;
				}
            			if (countflow==nflows[countk])
            			{
               				if (!strncmp(xline,"Update Data1:",13)) data1=1;
            			}
            			if (!strncmp(xline,"Mgen measure",12)) countflow++;
         		}
         		fseek(fm, 0L, 0);

         		first=0;
         		countflow=0;
         		
			while(!feof(fm))
         		{
            			fgets(xline,500,fm);
            			if (feof(fm)) {  
					break;
				}
            			if (!strncmp(xline,"Mgen measure",12))
            			{
               				if (countflow==nflows[countk])
               				{
                   				update_file();
                   				first=1;
               				}
               				countflow++;
            			}
            			fputs(xline,ft);
         		}
      		}
		if (first==0) {
			 update_file();
		}
      		fclose(ft);
      		fclose(fm);
      		remove(mname);
      		rename(xname,mname);
	
	}
}

//////////////////////////////////////////////////////////////////
//CHECK AND OPEN FILE .DLY AND GET IP VERSION
//////////////////////////////////////////////////////////////////

void  check_input_file(char *name)
{
   	int countflow;
   	char *ptr;
   	char counta,countb,countc;
   	FILE *fm;
   	char mname[300];
   	char xline[500];
	printf("\nFile to check: %s\n", name);
   	f=fopen(name,"r");
   
	if (f==NULL)
   	{
      		printf("\n MGEN - Distribution generator:\n\n ERROR: Input file not found.\n");
      		exit (-1);
   	}
   
	fgets(line,256,f);
   	ptr = strstr(line,"TxTime");
   
	// set the offset for the TxTIME for different type of files (IPv4, IPv6)
   	time_stamp_offset=ptr-line+7;
   	// look for the address type searching for @N.N.N.N
	counta=0;              
   
	for (countb=0; countb<30; countb++)
     		if (line[ptr-line-countb]=='.') counta++;
   		if (counta>=2) ipver='4';
   		else           ipver='6';
   
	printf("\n - detected for file %s flows with IP version: ipv%c",name,ipver);

   	//reading the packet size for specifed flow
   	//magic number
	read_flow=1977;
   
	while (read_flow!=fixed_flow)
   	{
      		fgets(line,256,f);
      		if (feof(f)){  
			break;
		}
      		strncpy(flownumber, &line[5], 4);
      		read_flow=atoi(flownumber);
   	}
   
	if (read_flow==fixed_flow)
   	{
      		ptr = strstr(line,"Size");
      		strncpy(size, &line[ptr-line+5], 20);
      		pack_size[countk]=atoi(size);
   	}

   	// Reading if available the title in the MCL file for CGRAPH file
   	strcpy(flow_title[countk],size);  // default title is pack. size
   	flow_title[countk][strlen(flow_title[countk])-1]=' ';

   	strcpy(mname,name);
   	mname[strlen(mname)-3]='m';
   	mname[strlen(mname)-2]='c';
   	mname[strlen(mname)-1]='l';
   	fm=fopen(mname,"r");
   
	if (fm==NULL) {
		printf("\n MGEN - Distribution generator:\n\n WARNING:  MCL file not found.\n");   
		exit(-1);
	}
	else
   	{
      		countflow=1;
      		
		while(!feof(fm))
      		{
         		fgets(xline,500,fm);
         
			if (feof(fm)){  
				break;
			}
         		if (!strncmp(xline,"Mgen measure",12))
         		{
            			if (countflow==fixed_flow)
            			{
               				fgets(xline,500,fm);
               				strncpy(flow_title[countk],&xline[11],strlen(line)-11);
               				flow_title[countk][strlen(xline)-11]='\0';
               				
					// filter  ONLY CR for GRAPH label style :)
               				for (countc=0; countc<strlen(flow_title[countk]); countc++)
                 				if ((flow_title[countk][countc]=='\n'))
                   					flow_title[countk][countc]='\0';
            			}
            			countflow++;
         		}
      		}
   	}
   	
	fclose(fm);
   	fseek(f, 0L, 0);

}

////////////////////////////////////////////////////////////////////////////////
//GENERATE COORD. USING DELAY FILES FOR OWD Average Distribution
////////////////////////////////////////////////////////////////////////////////

void process_ONEWAY_COMPARE(char *identificacion, struct distr_parameters *dparam)           
{

	int FX_factor;
	char aux[50];

   	sprintf(aux, "graphDAT%s", identificacion);

   	// re-make graphic range
   	old_gap=(max-min);
   	old_min=min-0.0002;
   	min=10000;
   	max=0.000001;
   
	for(countk=0; countk<fnum; countk++)
   	{
      		flow_min[countk]=flow_min[countk]-average[countk];
      		flow_max[countk]=flow_max[countk]-average[countk];
      		if (flow_max[countk]>max){  
			max=flow_max[countk];
		}
      		if (flow_min[countk]<min){  
			min=flow_min[countk];
   		}
	}
   
// -------------------------------------------------------
// in my humble opinion these is only for have the same % values to
// compare with the old one-way, using FX_factor...!!!
	max=max+old_gap-(max-min);
// -------------------------------------------------------

	//for better view
   
	max=max+0.0002;
   	min=min-0.0002;

   	if(strcmp(dparam->argv_min,"-")) { 
		min=(atof(dparam->argv_min))*0.001;
   	}
	if(strcmp(dparam->argv_max,"-")){ 
		max=(atof(dparam->argv_max))*0.001;
	}

   	// start graphic
   	gap=(max-min);
   	real_size=dparam->graphic_points;
   	real_step=gap/real_size;
   	open_all_temp_files();
   
	for(countf=0; countf<fnum; countf++)
   	{
      		FX_factor=(old_min-min-average[countf])/real_step;
      		while(!feof(faux[countf]))
      		{
   	 		fgets(line,256,faux[countf]);
	 		if (feof(faux[countf])){ 
				break;
			}
	 		delay=atof(line);
         		var=(delay-old_min)/real_step+FX_factor;
         		
			if ( (var>dparam->graphic_points) || (var<0) )
         		{
            			printf(" fatal RANGE_VALUE error.....!!!");
				//exit(-1);
         		}
         		else  dist[countf][var]++;
      		}
   	}
   	
	close_all_temp_files();
   	f=fopen(aux,"w");
   
	for (countp=0; countp<=dparam->graphic_points; countp++)
   	{
      		//TIME IN msec
		fprintf(f,"%.2f ", (min+countp*real_step)*1000);  
      
		//AXIS-Y IN %
		for(countf=0; countf<fnum; countf++) 
      		{
	 		fprintf(f,"%f ",100*dist[countf][countp]/count_dat[countf]);
      		}
      		fprintf(f,"\n");
   	}
   	fclose(f);
}

void delete_blanks(char *line)
{
	int lng;
	
	lng=strlen(line);
	
	line[lng-1]='\0';

	while (lng > 0 && (line[lng]=='\0' || line[lng]==' ')) {
		if (line[lng]==' ') {
			line[lng]='\0';
		}
		lng--;
	}
}


void err_function(int code)
{
	switch (code) {

		case 1:
			printf("\nError: Incorrect arguments\n");
			break;
		case 2:
			printf("\nError: Too much input files\n");
			break;		
		
		case 3:
			printf("\nError: Incorrect number of arguments\n");
			printf(" MGEN - Distribution generator:\nUsage: distr min max opt [0-4]  AnalisiType [1-2]  OutFile  size  InFile1 Flow .. InFileN Flow \n");
			printf("Usage: distr -f <arguments file>\n");

			break;
		case 4: 
			printf("\nError: bad arguments file\n");
			break;

		case 5:
			printf("\nError opening arguments file\n");
			break;
		
		case 6:
			printf("\nError: arguments file is empty\n");
			break;

		default:
			printf("\nError executing distr\n");
			break;
	}
	exit(-1);
}


struct distr_parameters get_params(char *argv[], int argc)
{
	struct distr_parameters param;
	int lng, count_f;	
	FILE *f_param;
	char file_conf[50];
	char line[300];
	int count_pos_file=0;
	int count_pos_file_aux=0;
	char *cad;
	int long_aux;

	if (argc!=3 && (argc < first_file+1)) {
		
		err_function(3);
	}
	
	if (strcmp(argv[1],"-f"	)==0) {
	
		printf("\nArguments by file\n");
		if (strlen(argv[2]) > 50) {
			err_function(4);
		}
		f_param=fopen(argv[2],"r");
		if (!f_param) {
			err_function(5);
		}
		
		fgets(line,300,f_param);
		if (feof(f_param)) {
			err_function(6);
		}		

		
		
		while (!feof(f_param)) {
			
			delete_blanks(line);
			
			if( (cad=strstr(line,"min_time"))!=NULL) {
				long_aux=strlen("min_time=");
				cad=cad+long_aux;			
				if (strlen(cad) > 20) {
                                        err_function(1);
                                }
                                strcpy(param.argv_min, cad);				
			}
			else if( (cad=strstr(line,"max_time"))!=NULL) {
                                long_aux=strlen("max_time=");
                                cad=cad+long_aux;
                                if (strlen(cad) > 20) {
                                        err_function(1);
                                }
                                strcpy(param.argv_max, cad);
                        }
			else if( (cad=strstr(line,"option"))!=NULL) {
                                long_aux=strlen("option=");
                                cad=cad+long_aux;
                                if (strlen(cad) > 2) {
                                        err_function(1);
                                }
                                param.opt=cad[0];
                        }
			else if( (cad=strstr(line,"transitory"))!=NULL) {
                                long_aux=strlen("transitory=");
                                cad=cad+long_aux;
                                if (strlen(cad) > 2) {
                                        err_function(1);
                                }
                                param.trans=cad[0];
                        }
			else if( (cad=strstr(line,"graph_name"))!=NULL) {
                                long_aux=strlen("graph_name=");
                                cad=cad+long_aux;
                                if (strlen(cad) > 300) {
                                        err_function(1);
                                }
                                strcpy(param.out_name, cad);
                        }

			else if( (cad=strstr(line,"xsize"))!=NULL) {
                                long_aux=strlen("xsize=");
                                cad=cad+long_aux;
                                if (strlen(cad) > 5) {
                                        err_function(1);
                                }
                                param.graphic_points=atoi(cad);
                        }
			
			else if( (cad=strstr(line,"pixel_gap"))!=NULL) {
                                long_aux=strlen("pixel_gap=");
                                cad=cad+long_aux;
                                if (strlen(cad) > 10) {
                                        err_function(1);
                                }
                                strcpy(param.esp, cad);
                        }
			else if( (cad=strstr(line,"text_gap"))!=NULL) {
                                long_aux=strlen("text_gap=");
                                cad=cad+long_aux;
                                if (strlen(cad) > 10) {
                                        err_function(1);
                                }
                                strcpy(param.xes, cad);
                        }

			else if( (cad=strstr(line,"ysize"))!=NULL) {
                                long_aux=strlen("ysize=");
                                cad=cad+long_aux;
                                if (strlen(cad) > 10) {
                                        err_function(1);
                                }
                                strcpy(param.ty, cad);
                        }

			else if( (cad=strstr(line,"fill_graphic"))!=NULL) {
                                long_aux=strlen("fill_graphic=");
                                cad=cad+long_aux;
                                if (strlen(cad) > 2) {
                                        err_function(1);
                                }
                                param.rfill=cad[0];
                        }
			
			else if( (cad=strstr(line,"zero_reference"))!=NULL) {
                                long_aux=strlen("zero_reference=");
                                cad=cad+long_aux;
                                if (strlen(cad) > 2) {
                                        err_function(1);
                                }
                                param.zrefx=cad[0];
                        }
	
			else if( (cad=strstr(line,"extra_layout"))!=NULL) {
                                long_aux=strlen("extra_layout=");
                                cad=cad+long_aux;
                                if (strlen(cad) > 2) {
                                        err_function(1);
                                }
                                param.rback=cad[0];
                        }
			else if( (cad=strstr(line,"enable_pattern"))!=NULL) {
                                long_aux=strlen("enable_pattern=");
                                cad=cad+long_aux;
                                if (strlen(cad) > 10) {
                                        err_function(1);
                                }
                                strcpy(param.rpattern, cad);
                        }

			
			else {
				if(count_pos_file_aux % 2 == 0) {
					
					strcpy(param.fnames[count_pos_file], line);
					
				}
				else {
					param.nflows[count_pos_file]=atoi(line);
					count_pos_file++;
				}
				count_pos_file_aux++;
			}			
			
			fgets(line,300,f_param);	

		
		} //END WHILE
		fnum=(count_pos_file);
		
        	if (fnum > MAX_INPUT_FILES) {
                	err_function(2);
        	}

	
	}
	else {
			
		if (argc < first_file+1) {

                	err_function(3);
        	}


		printf("\nArguments by command line\n");
		
		fnum=(argc-first_file)/2;
	        if (fnum > MAX_INPUT_FILES) {
                	err_function(2);
        	}


		if (strlen(argv[1]) > 20) {
			err_function(1);
		}
		strcpy(param.argv_min, argv[1]);
		
		
		if (strlen(argv[2]) > 20) {
			err_function(1);
		}
		strcpy(param.argv_max, argv[2]);

		if (strlen(argv[3]) > 1) {
			err_function(1);
		}
		param.opt=*argv[3];
		
		if (strlen(argv[4]) > 1) {
			err_function(1);
		} 
		param.trans=*argv[4];

		if (strlen(argv[5]) > 300) {
			err_function(1);
		}
		strcpy(param.out_name, argv[5]);

		if (strlen(argv[6]) > 5) {
			err_function(1);
		}
		param.graphic_points=atoi(argv[6]);
	
		if (strlen(argv[7]) > 10) {
			err_function(1);
		}
		strcpy(param.esp, argv[7]);

		if (strlen(argv[8]) > 10) {
                        err_function(1);
                }
		strcpy(param.xes, argv[8]);

		if (strlen(argv[9]) > 10) {
                        err_function(1);
                }
		strcpy(param.ty, argv[9]);

		if (strlen(argv[10]) > 1 || (*argv[10]!='0' && *argv[10]!='1')) {
                        err_function(1);
                }
		param.rfill=*argv[10];
		
		if (strlen(argv[11]) > 1 || (*argv[11]!='0' && *argv[11]!='1') ) {
                        err_function(1);
                }
		param.zrefx=*argv[11];
		
		if (strlen(argv[12]) > 1 || (*argv[12]!='0' && *argv[12]!='1')) {
			err_function(1);
                }
		param.rback=*argv[12];

		if (strlen(argv[13]) > 10) {
                        err_function(1);
                }
		strcpy(param.rpattern, argv[13]);

		

		for(count_f=0; count_f<fnum; count_f++)
        	{
                	strcpy(param.fnames[count_f],argv[first_file+count_f*2]);
                	param.nflows[count_f]=atoi(argv[first_file+count_f*2+1]);
        		
		}


	}
	return param;
}





// ########################################################################
//                       M     A       I       N
// ########################################################################


int main (int argc, char *argv[])
{
	int i, time_stamp_plus1, pid_aux, maxkey, num_secuence;
	float retard, aux_last;
	Arbol Arbolf=NULL;
	time_t tiempo;
	char identificacion[50], aux_conf[50], aux_conf2[50], aux_conf3[50], direct_act[60], aux_exec[50], aux_ident[50];   	
	struct distr_parameters dparam;

   	dparam=get_params(argv, argc);

   	max_SeqNum=0;
   	min_SeqNum=9999;
   	min=10000;
   	max=0.000001;
   	strcpy(proc,"timeN_f0");
   	fout_name=&proc[0];

   	for(countf = 0; countf < fnum; countf++) {
      		flow_max[countf]=0.000001;
      		flow_min[countf]=10000;
   	}
   	for(countp = 0; countp <= dparam.graphic_points; countp++) 
      		for(countf=0; countf<fnum; countf++) 
       			dist[countf][countp]=0;
   			for(countp=0; countp<fnum; countp++) {
       				faux[countp]=NULL;
       				//pack_size de cada fichero
				pack_size[countp]=0;   
       				//numero de mostras para cada punto
				count_dat[countp]=0;   
       				average[countp]=0;
       				last_SeqNum[countf]=0;
 		  	}
		
	
	printf("\n MGEN - DISTribution generator for Netmeter v2.0 \n\n");
        printf("\n list of parameters:");
        printf("\n - min = %s",dparam.argv_min);
        printf("\n - max = %s",dparam.argv_max);
        printf("\n - task option = %c", dparam.opt);
        printf("\n - transistory proces = %c", dparam.trans);
        printf("\n - output files path  = %s/",DIRECTORY_FILE_OUT);
        printf("\n - output image name  = ./%s", dparam.out_name);
        printf("\n - graph size (pixel) = %d", dparam.graphic_points);
        printf("\n - ESP                = %s", dparam.esp);
        printf("\n - XES                = %s", dparam.xes);
        printf("\n - TY                 = %s", dparam.ty);
        printf("\n - Fill               = %c", dparam.rfill);
        printf("\n - Zero               = %c", dparam.zrefx );
        printf("\n - Layout             = %c", dparam.rback);
        printf("\n - Pattern            = %s", dparam.rpattern);
	printf("\n - Number of files	= %d", fnum);
 	printf("\n\n list of input files [with specifed fixed flow]");
  	 
	for(countk=0; countk<fnum; countk++)
   	{
     		printf("\n - input file name = %s  ",dparam.fnames[countk]);
     		printf("\n - flow number     = %d  ",dparam.nflows[countk]);
   	}

	

   	//SAVE DIRECTORY ACTUAL FOR EXECUTE graph2002
	getcwd(direct_act,sizeof(char)*60);
	
	if (chdir(DIRECTORY_FILE_OUT))
   	{
      		mkdir(DIRECTORY_FILE_OUT);
      		chdir(DIRECTORY_FILE_OUT);
   	}
   

   	///////////////////////////////////////
        /// FILE UNIQUE IDENTIFICACION  ///////

        pid_aux=getpid();
        tiempo = time( NULL );

        sprintf(identificacion, "_%u_%d_option%c", tiempo, pid_aux, dparam.opt);

        //
	///////////////////////////////////////


	//FOR IPDV.........
	if (dparam.opt=='7')        
   	{
      		maxf=0;
      		minf=0;
      		if (strcmp(dparam.argv_min,"-")) {  
			maxf=1; min=(atof(dparam.argv_min))*0.001; 
		}
      
		if (strcmp(dparam.argv_max,"-")) {  
			minf=1; max=(atof(dparam.argv_max))*0.001; 
		}
     		sprintf(aux_ident, "graphDAT%s", identificacion); 
		fout=fopen(aux_ident,"w");

   	}

   	time_stamp_plus1 = 0;
   
	//MAIN LOOP TO GET THE FILES

	for(countk=0; countk<fnum; countk++)
   	{
      		last_SeqNum[countk] = 0;
      		last=0;
      		total_delay=0;
      		fixed_flow=dparam.nflows[countk];
      		
		check_input_file(dparam.fnames[countk]);
      
      		proc[4]= (char)(countk + '0');
		
		////////////////////////////////////////////////////////////
		//CONCAT THE IDENTIFICACION 0N FILE NAME		
		if (countk == 0) {
			strcat(proc, identificacion);
			
		}
		 	
		//////////////////////////////////////////////////////////

      		if (dparam.opt!='7') {        
        		//Generate k files "procK"
			fout=fopen(fout_name,"w");   
		}
      		
		if (dparam.trans=='0') {
			for(count=0; count<TRANSITORY_LINES; count++) {
		      		fgets(line,256,f);
      		      		     
      			}
      		}
      
		if (time_stamp_plus1) {
	      		time_stamp_plus1=0;
	      		time_stamp_offset--;
     		}

		//AVL TO SAVE THE NUM OF SECUENCE AND DELAY OF THE PACKETS ON THE CURRENT .dly
		Arbolf=NULL;

		while(!feof(f))
      		{
      	 
			fgets(line,256,f);
     	 		
			if (feof(f)) {
		 		break;
        		}
			//show de current packet
			//printf("Processing: %s\n", line);	
			strncpy(flownumber, &line[5], 4);
         
			read_flow=atoi(flownumber);
			
			if (read_flow==fixed_flow)
         		{
	     			strncpy(seq,  &line[14],   8);
	     			seq[8]='\0';
             			SeqNum=atoi(seq);
	     
	     		
				if (SeqNum > 1000000 && !time_stamp_plus1) {
		     			time_stamp_offset++;
		     			time_stamp_plus1 = 1;
	     			}

             			strncpy(hor1, &line[time_stamp_offset],   2);
	     			strncpy(min1, &line[time_stamp_offset+3], 2);
             			strncpy(sec1, &line[time_stamp_offset+6], 9);
             			strncpy(hor2, &line[time_stamp_offset+23],2);
             			strncpy(min2, &line[time_stamp_offset+26],2);
             			strncpy(sec2, &line[time_stamp_offset+29],9);
	     			
             			min1[2]='\0';
	     			hor1[2]='\0';
             			sec1[9]='\0';
	     			min2[2]='\0';
	     			hor2[2]='\0';
             			sec2[9]='\0';

             			temp=(atoi(hor2)-atoi(hor1))*3600 +(atoi(min2)-atoi(min1))*60;
             			_sec1=atof(sec2)-atof(sec1);
             			delay=_sec1+temp;
             		
				//ONE WAY AVERAGE DISTRIBUTION....
				if (dparam.opt=='6') 
             			{

					if (delay<min) { min=delay; }
	        			if (delay>max) { max=delay; }
	        			if (delay<flow_min[countk] ) { flow_min[countk]=delay; }
	        			if (delay>flow_max[countk] )  { flow_max[countk]=delay; }
   	        		
					//INSERT AVL
				
                       
					Insertar(&Arbolf, SeqNum, delay);
					
					count_dat[countk]++;
					
             			}
             			//IPDV ...
				else if (dparam.opt=='7') {     
             			
					Insertar(&Arbolf, SeqNum, delay);
				
				}
				//ONE WAY DELAY DISTRIBUTION // ONE WAY DELAY...
				else if (dparam.opt == '0' || dparam.opt == '4') { 
			
					if (delay<min) {
						min=delay;
					}
					if (delay>max) {
						max=delay;
					}
			
					
					Insertar(&Arbolf, SeqNum, delay);
					count_dat[countk]++;

					
	     			}
             			//IPDV DISTRIBUTION ..........
				else if (dparam.opt=='1') {      
             
					Insertar(&Arbolf, SeqNum, delay);

				}
             			
				//UPDATE_MCL
				else {               
             		
	        			count_dat[countk]++;
   	        			fprintf(fout,"%f\n",delay);
             			}
 	     
				total_delay=total_delay+delay; // expressed in msec
	     		
             			last=delay;
             			last_SeqNum[countk] = SeqNum;
         		}
		}// END OF THE LOOP THAT GETS ALL THE LINES 
      		
		/////////////////////////////////////////////////////////////////
		//WRITE CORRECTLY TEMP FILES WITH AVL HELP
		/////////////////////////////////////////////////////////////////

		
		if (dparam.opt=='0' || dparam.opt=='4') {

			

			
			maxkey=ClaveMaxima(Arbolf);
			
			for(num_secuence=0; num_secuence <= maxkey; num_secuence++) {
				retard= Buscar(Arbolf, num_secuence);
				
				//THE PACKET IS LOST
				if (retard == -1) {
					
					//IF OWD PRINT "time 0.0"
					if (dparam.opt=='4') {
						fprintf(fout,"%f\n", 0.0);
					}
				}
				
				else {
					fprintf(fout,"%f\n", retard);
				}
			}
			//show the computes that have been saved in memory
			//InOrden(Arbolf, Mostrar);
			Podar(&Arbolf);
		}
		//OWD Average Distribution
		if (dparam.opt=='6') {
			
			maxkey=ClaveMaxima(Arbolf);

                        for(num_secuence=0; num_secuence <= maxkey; num_secuence++) {
                                retard= Buscar(Arbolf, num_secuence);

                                //ONLY THE PACKETS THAT AREN'T LOST
                                if (retard != -1) {
					 fprintf(fout,"%f\n", retard);

                                }

                        }
			Podar(&Arbolf);
		}
		
		
		//IPDV DISTRIBUTION, IPDV
		if (dparam.opt=='1' || dparam.opt=='7') {
			
			maxkey=ClaveMaxima(Arbolf);
                        aux_last=Buscar(Arbolf, 0);
			for(num_secuence=1; num_secuence <= maxkey; num_secuence++) {
                                
				retard= Buscar(Arbolf, num_secuence);
                                
				if (retard != -1 && aux_last != -1) {
                                      ipdv=retard-aux_last;
					
					//IPDV DISTRIBUTION
					if (dparam.opt=='1') {
	        
							fprintf(fout,"%f\n", ipdv);
                                			if ((ipdv<-0.5) || (ipdv>100)) {
                                                		ipdv=ipdv;
                                        		}
                                        		if (ipdv<min) {
                                        			min=ipdv;
                                        		}
                                        		if (ipdv>max) {
                                        			max=ipdv;
                                      		}
							count_dat[countk]++;
					}
					//IPDV
					else { 
						if  (( (minf==0) || (ipdv>=min) ) && ( (maxf==0) || (ipdv<=max) )) {
							fprintf(fout,"%d %f \n",num_secuence,ipdv);
						}			
		
					}
				}
				else {
					
				}
				aux_last=retard;
			}
			//InOrden(Arbolf, Mostrar);
			Podar(&Arbolf);	
		}
	
		
		else {


		}	
		
		fclose(f);
      
		//IPDV	
		if (dparam.opt=='7')        
      		{
         
			fprintf(fout,"NEW\n");
         		average[countk]=total_delay/count_dat[countk];
         		//printf("\n  pack_size %d - average delay (msec *1000) %d: %f\n",pack_size[countk],average[countk]);
		}
      
		else
      		{
         		fclose(fout);
         		average[countk]=total_delay/count_dat[countk];
         		//printf("\n  pack_size %d - average delay (sec): %f \n",pack_size[countk],average[countk]);
      		}
   	
	} // END OF THE LOOP THAT GETS THE FILES .DLY
   		
	//IPDV
	if (dparam.opt=='7') {       
   		fclose(fout);
   	}


   	if (dparam.opt!='6') {
   	
      		if(strcmp(dparam.argv_min,"-")) min=(atof(dparam.argv_min))*0.001;
      		if(strcmp(dparam.argv_max,"-")) max=(atof(dparam.argv_max))*0.001;
   	}
   
	// CALL TO THE FUNCTIONS TO CALCULATE THE GRAPHIC POINTS USING TIMEN_F0..

   	if      (dparam.opt=='4')  process_HEAD(identificacion, &dparam);
   	else if (dparam.opt=='5')  process_UPDATE_MCL();
   	else if (dparam.opt=='6')  process_ONEWAY_COMPARE(identificacion, &dparam);
   	else if (dparam.opt=='7')  ;
   	else                process_IPDV_ONEWAY(identificacion, &dparam);

	//DELETE TEMP FILES TIME0_F0 .... FROM /TMP
	//
	//delete_temp(identificacion);
	//	
	//////////////////////////////////////////



/////////////////////////////////////////////////////////////////////////
//   MAKE file "cgraf"  GRAPHIC CONFIGURATION
/////////////////////////////////////////////////////////////////////////

	sprintf(aux_conf, "cgraph%s", identificacion);
	f=fopen(aux_conf,"w");
	fprintf(f,"ESP %s\n",dparam.esp);
	fprintf(f,"XES %s\n",dparam.xes);
	fprintf(f,"TY %s\n",dparam.ty);

	//   if ((opt=='0') || (opt=='1') ||(opt=='7') || (opt=='6'))  // One-way x 2 + IPDV
	{   
      		if (dparam.rfill=='1') fprintf(f,"RFILL ON\n");
      		if (dparam.rback=='1') fprintf(f,"RBACK ON\n");
      		if (strcmp(dparam.rpattern,"-")!=0) {
			 fprintf(f,"RPATTERN %s\n",dparam.rpattern);
		}
	}
   
	if ((dparam.opt=='1') || (dparam.opt=='6') )     // IPDV distribution + ONE WAY compare
   	{   
      		if (dparam.zrefx=='1') fprintf(f,"RZERO ON\n");
   	}       
   
	if (dparam.opt=='7')      // IPDV   
   	{   
       		fprintf(f,"TYPE X\n");                // the my X-type  ;)
       		fprintf(f,"RXSIZE %d\n",dparam.graphic_points);
       		fprintf(f,"RYSIZE %s\n",dparam.ty);
   	}       
   	else fprintf(f,"TYPE N\n");
   	
	fprintf(f,"Lab 0\n");
   	fprintf(f,"Font FS\n");
   	fprintf(f,"Output %s\n",dparam.out_name);
   	fprintf(f,"Input %s",DIRECTORY_FILE_OUT);
   
   	sprintf(aux_conf3,"/graphDAT%s\n", identificacion);
   	fprintf(f, aux_conf3);
   
	//fprintf(f,"/graphDAT\n");
   	if (dparam.opt=='7') 
   	{   
      		fprintf(f,"Dat 1");
   	}
   	else
   	{
      		fprintf(f,"Dat ");
      		
		for (countp=1; countp<=fnum;countp++) fprintf(f," %d",countp);
   	}      
   	fprintf(f,"\nLegend Este FM ");
   	for (countp=0; countp<fnum; countp++)
   	{
      
		if (dparam.opt=='6') {
         		sprintf(line,"+Average delay (msec): %.2f",average[countp]*1000);
         		strcat(flow_title[countp],line);
      		}
      		fprintf(f," ?%s!",flow_title[countp]);
   	}
   
	fprintf(f,"\nTitle Oeste FM %s",Ytitle[dparam.opt-'0']);
   	fprintf(f,"\nTitle Sur FM %s",Xtitle[dparam.opt-'0']);
   	fprintf(f,"\nEJE flt");
   	fprintf(f,"\nTitle Norte FM %s",task[dparam.opt-'0']);
   	
	fclose(f);

   	
	//PUT ON NETMETER FOLDER
	chdir(direct_act);
	
	
   
   	////////////////////////////////////////////
   	//EXECUTE graph2002
   	//Graph Command: bin/graph2002 -cfg temp/cgraph
     	//EXECUTABLE NAME AND PATH
	
	sprintf(aux_exec, "%s/bin/graph2002", direct_act);
	
	//NAME OF CONF FILE AND PATH
	sprintf(aux_conf2,"/tmp/%s", aux_conf);
	
	printf("orden de ejecucion: %s\n", aux_exec);
	//printf("camino temporal: %s\n", aux_conf2);
	
   	if (execl(aux_exec,"graph2002","-cfg",aux_conf2,NULL) < 0) {
     		printf("Can't execute graph2002\n");
   	}
   	////////////////////////////////////////////

}

