/* Arbol AVL en C                      */

#include <stdlib.h>
#include <stdio.h>
#include "arbolAVLDouble.h"
#define TRUE 1
#define FALSE 0

enum {IZQUIERDO, DERECHO};

void Equilibrar(ArbolDouble *raiz, pNodoDouble nodo, int, int);
void RSI(ArbolDouble *raiz, pNodoDouble nodo);
void RSD(ArbolDouble *raiz, pNodoDouble nodo);
void RDI(ArbolDouble *raiz, pNodoDouble nodo);
void RDD(ArbolDouble *raiz, pNodoDouble nodo);

/* Funciones auxiliares: */
void Podar(ArbolDouble *a);
void auxContador(ArbolDouble a, int*);
void auxAltura(ArbolDouble a, int, int*);

/* Poda: borrar todos los nodos a partir de uno, incluido */
void Podar(ArbolDouble *a)
{
   /* Algoritmo recursivo, recorrido en postorden */
   if(*a) {
      Podar(&(*a)->izquierdo); /* Podar izquierdo */
      Podar(&(*a)->derecho);   /* Podar derecho */
      free(*a);                /* Eliminar nodo */
      *a = NULL;
   }
}

/* Insertar un dato en el rbol ABB */
void InsertarDouble(ArbolDouble *a, double dat)
{
   pNodoDouble padre = NULL;
   pNodoDouble actual = *a;

   /* Buscar el dato en el rbol, manteniendo un puntero al nodo padre */
   while(!VacioDouble(actual) && dat != actual->dato) {
      padre = actual;
      if(dat < actual->dato)
	actual = actual->izquierdo;
      else if (dat > actual->dato)
	actual = actual->derecho;
   }

   /* Si se ha encontrado el elemento, se actualiza el contenido */
   if(!VacioDouble(actual)) {
	actual->ocurrencias++;
	return;
   }
   /* Si padre es NULL, entonces el rbol estaba vaco, el nuevo nodo ser
      el nodo raiz */
   if(VacioDouble(padre)) {
      *a = (ArbolDouble)malloc(sizeof(tipoNodoDouble));
      (*a)->dato = dat;
      (*a)->ocurrencias = 1;
      (*a)->izquierdo = (*a)->derecho = NULL;
      (*a)->padre = NULL;
      (*a)->FE = 0;
   }
   /* Si el dato es menor que el que contiene el nodo padre, lo insertamos
      en la rama izquierda */
   else if(dat < padre->dato) {
      actual = (ArbolDouble)malloc(sizeof(tipoNodoDouble));
      padre->izquierdo = actual;
      actual->dato = dat;
      actual->ocurrencias = 1;
      actual->izquierdo = actual->derecho = NULL;
      actual->padre = padre;
      actual->FE = 0;
      Equilibrar(a, padre, IZQUIERDO, TRUE);
   }
   /* Si el dato es mayor que el que contiene el nodo padre, lo insertamos
      en la rama derecha */
   else if(dat > padre->dato) {
      actual = (ArbolDouble)malloc(sizeof(tipoNodoDouble));
      padre->derecho = actual;
      actual->dato = dat;
      actual->ocurrencias = 1;
      actual->izquierdo = actual->derecho = NULL;
      actual->padre = padre;
      actual->FE = 0;
      Equilibrar(a, padre, DERECHO, TRUE);
  }
}

/* Equilibrar rbol AVL partiendo del nodo nuevo */
void Equilibrar(ArbolDouble *a, pNodoDouble nodo, int rama, int nuevo)
{
   int salir = FALSE;

   /* Recorrer camino inverso actualizando valores de FE: */
   while(nodo && !salir) {
      if(nuevo)
         if(rama == IZQUIERDO) nodo->FE--; /* Depende de si aadimos ... */
         else                  nodo->FE++;
      else
         if(rama == IZQUIERDO) nodo->FE++; /* ... o borramos */
         else                  nodo->FE--;
      if(nodo->FE == 0) salir = TRUE; /* La altura de las rama que
                                         empieza en nodo no ha variado,
                                         salir de Equilibrar */
      else if(nodo->FE == -2) { /* Rotar a derechas y salir: */
         if(nodo->izquierdo->FE == 1) RDD(a, nodo); /* Rotacin doble  */
         else RSD(a, nodo);                         /* Rotacin simple */
         salir = TRUE;
      }
      else if(nodo->FE == 2) {  /* Rotar a izquierdas y salir: */
         if(nodo->derecho->FE == -1) RDI(a, nodo); /* Rotacin doble  */
         else RSI(a, nodo);                        /* Rotacin simple */
         salir = TRUE;
      }
      if(nodo->padre)
         if(nodo->padre->derecho == nodo) rama = DERECHO; else rama = IZQUIERDO;
      nodo = nodo->padre; /* Calcular FE, siguiente nodo del camino. */
   }
}

/* Rotacin doble a derechas */
void RDD(ArbolDouble *raiz, ArbolDouble nodo)
{
   pNodoDouble Padre = nodo->padre;
   pNodoDouble P = nodo;
   pNodoDouble Q = P->izquierdo;
   pNodoDouble R = Q->derecho;
   pNodoDouble B = R->izquierdo;
   pNodoDouble C = R->derecho;

   if(Padre)
     if(Padre->derecho == nodo) Padre->derecho = R;
     else Padre->izquierdo = R;
   else *raiz = R;

   /* Reconstruir rbol: */
   Q->derecho = B;
   P->izquierdo = C;
   R->izquierdo = Q;
   R->derecho = P;

   /* Reasignar padres: */
   R->padre = Padre;
   P->padre = Q->padre = R;
   if(B) B->padre = Q;
   if(C) C->padre = P;

   /* Ajustar valores de FE: */
   switch(R->FE) {
      case -1: Q->FE = 0; P->FE = 1; break;
      case 0:  Q->FE = 0; P->FE = 0; break;
      case 1:  Q->FE = -1; P->FE = 0; break;
   }
   R->FE = 0;
}

/* Rotacin doble a izquierdas */
void RDI(ArbolDouble *a, pNodoDouble nodo)
{
   pNodoDouble Padre = nodo->padre;
   pNodoDouble P = nodo;
   pNodoDouble Q = P->derecho;
   pNodoDouble R = Q->izquierdo;
   pNodoDouble B = R->izquierdo;
   pNodoDouble C = R->derecho;

   if(Padre)
     if(Padre->derecho == nodo) Padre->derecho = R;
     else Padre->izquierdo = R;
   else *a = R;

   /* Reconstruir rbol: */
   P->derecho = B;
   Q->izquierdo = C;
   R->izquierdo = P;
   R->derecho = Q;

   /* Reasignar padres: */
   R->padre = Padre;
   P->padre = Q->padre = R;
   if(B) B->padre = P;
   if(C) C->padre = Q;

   /* Ajustar valores de FE: */
   switch(R->FE) {
      case -1: P->FE = 0; Q->FE = 1; break;
      case 0:  P->FE = 0; Q->FE = 0; break;
      case 1:  P->FE = -1; Q->FE = 0; break;
   }
   R->FE = 0;
}

/* Rotacin simple a derechas */
void RSD(ArbolDouble *a, pNodoDouble nodo)
{
   pNodoDouble Padre = nodo->padre;
   pNodoDouble P = nodo;
   pNodoDouble Q = P->izquierdo;
   pNodoDouble B = Q->derecho;

   if(Padre)
     if(Padre->derecho == P) Padre->derecho = Q;
     else Padre->izquierdo = Q;
   else *a = Q;

   /* Reconstruir rbol: */
   P->izquierdo = B;
   Q->derecho = P;

   /* Reasignar padres: */
   P->padre = Q;
   if(B) B->padre = P;
   Q->padre = Padre;

   /* Ajustar valores de FE: */
   P->FE = 0;
   Q->FE = 0;
}

/* Rotacin simple a izquierdas */
void RSI(ArbolDouble *a, pNodoDouble nodo)
{
   pNodoDouble Padre = nodo->padre;
   pNodoDouble P = nodo;
   pNodoDouble Q = P->derecho;
   pNodoDouble B = Q->izquierdo;

   if(Padre)
     if(Padre->derecho == P) Padre->derecho = Q;
     else Padre->izquierdo = Q;
   else *a = Q;

   /* Reconstruir rbol: */
   P->derecho = B;
   Q->izquierdo = P;

   /* Reasignar padres: */
   P->padre = Q;
   if(B) B->padre = P;
   Q->padre = Padre;

   /* Ajustar valores de FE: */
   P->FE = 0;
   Q->FE = 0;
}

/* Eliminar un elemento de un rbol ABB */
void BorrarDouble(ArbolDouble *a, double dat)
{
   pNodoDouble padre = NULL;
   pNodoDouble actual;
   pNodoDouble nodo;
   int aux;

   actual = *a;
   /* Mientras sea posible que el valor est en el rbol */
   while(!VacioDouble(actual)) {
      if(dat == actual->dato) { /* Si el valor est en el nodo actual */
         if(EsHojaDouble(actual)) { /* Y si adems es un nodo hoja: lo borramos */
            if(padre) /* Si tiene padre (no es el nodo raiz) */
               /* Anulamos el puntero que le hace referencia */
               if(padre->derecho == actual) padre->derecho = NULL;
               else if(padre->izquierdo == actual) padre->izquierdo = NULL;
            free(actual); /* Borrar el nodo */
            actual = NULL;
            /* El nodo padre del actual puede ser ahora un nodo hoja, por lo
               tanto su FE es cero, pero debemos seguir el camino a partir de
               su padre, si existe. */
            if((padre->derecho == actual && padre->FE == 1) ||
               (padre->izquierdo == actual && padre->FE == -1)) {
               padre->FE = 0;
               actual = padre;
               padre = actual->padre;
            }
            if(padre)
               if(padre->derecho == actual) Equilibrar(a, padre, DERECHO, FALSE);
               else                         Equilibrar(a, padre, IZQUIERDO, FALSE);
            return;
         }
         else { /* Si el valor est en el nodo actual, pero no es hoja */
            padre = actual;
            /* Buscar nodo ms izquierdo de rama derecha */
            if(actual->derecho) {
               nodo = actual->derecho;
               while(nodo->izquierdo) {
                  padre = nodo;
                  nodo = nodo->izquierdo;
               }
            }
            /* O buscar nodo ms derecho de rama izquierda */
            else {
               nodo = actual->izquierdo;
               while(nodo->derecho) {
                  padre = nodo;
                  nodo = nodo->derecho;
               }
            }

	    aux = actual->dato;
            actual->dato = nodo->dato;
            nodo->dato = aux;
            actual = nodo;
	}
      }
      else { /* Todava no hemos encontrado el valor, seguir buscndolo */
         padre = actual;
         if(dat > actual->dato) actual = actual->derecho;
         else if(dat < actual->dato) actual = actual->izquierdo;
      }
   }
}

/* Recorrido de rbol en inorden, aplicamos la funcin func, que tiene
   el prototipo:
   void func(int*);
*/
void InOrdenDouble(ArbolDouble a, void (*func)(double*))
{
   if(VacioDouble(a)) {
	return;
   }
   else {

   	if(a->izquierdo) InOrdenDouble(a->izquierdo, func);
   	func(&(a->dato));
        if(a->derecho) InOrdenDouble(a->derecho, func);
   }	
}

/* Recorrido de rbol en preorden, aplicamos la funcin func, que tiene
   el prototipo:
   void func(int*);
*/
void PreOrdenDouble(ArbolDouble a, void (*func)(double*))
{
   if(VacioDouble(a)) {
	return;
   }
   else {
   	func(&(a->dato));
   	if(a->izquierdo) PreOrdenDouble(a->izquierdo, func);
   	if(a->derecho) PreOrdenDouble(a->derecho, func);
   }	
}

/* Recorrido de rbol en postorden, aplicamos la funcin func, que tiene
   el prototipo:
   void func(int*);
*/
void PostOrdenDouble(ArbolDouble a, void (*func)(double*))
{
   if(VacioDouble(a)) {
   	return;
   }
   else {
   	if(a->izquierdo) PostOrdenDouble(a->izquierdo, func);
   	if(a->derecho) PostOrdenDouble(a->derecho, func);
   	func(&(a->dato));
   }
}

/* Contar el nmero de nodos */
int NumeroNodosDouble(ArbolDouble a, int *contador)
{
   *contador = 0;

   auxContador(a, contador); /* Funcin auxiliar */
   return *contador;
}

/* Funcin auxiliar para contar nodos. Funcin recursiva de recorrido en
   preorden, el proceso es aumentar el contador */
void auxContador(ArbolDouble nodo, int *c)
{
   (*c)++; /* Otro nodo */
   /* Continuar recorrido */
   if(nodo->izquierdo) auxContador(nodo->izquierdo, c);
   if(nodo->derecho)   auxContador(nodo->derecho, c);
}


//Busca la clave mas alta del rbol
int ClaveMaximaDouble(ArbolDouble a)
{
   
   pNodoDouble actual = a;
   pNodoDouble anterior = NULL;
   if (VacioDouble(actual)) {
	return -1;
   }
   else {

   	while (!VacioDouble(actual)) {
		anterior = actual;
		actual = actual->derecho;
   	}
   }	
   return anterior->dato;

}

/* Calcular la altura del rbol, que es la altura del nodo de mayor altura. */
int AlturaArbolDouble(ArbolDouble a, int *altura)
{
   *altura = 0;

   auxAltura(a, 0, altura); /* Funcin auxiliar */
   return *altura;
}

/* Funcin auxiliar para calcular altura. Funcin recursiva de recorrido en
   postorden, el proceso es actualizar la altura slo en nodos hojas de mayor
   altura de la mxima actual */
void auxAltura(pNodoDouble nodo, int a, int *altura)
{
   /* Recorrido postorden */
   if(nodo->izquierdo) auxAltura(nodo->izquierdo, a+1, altura);
   if(nodo->derecho)   auxAltura(nodo->derecho, a+1, altura);
   /* Proceso, si es un nodo hoja, y su altura es mayor que la actual del
      rbol, actualizamos la altura actual del rbol */
   if(EsHojaDouble(nodo) && a > *altura) *altura = a;
}

/* Comprobar si un rbol es vaco */
int VacioDouble(ArbolDouble r)
{
   return r==NULL;
}

/* Comprobar si un nodo es hoja */
int EsHojaDouble(pNodoDouble r)
{
   return !r->derecho && !r->izquierdo;
}

