/* ####################################################
   # Copyright 2002, 2003 Ren Serral,                #
   #                                                  #
   # common.h - Library for common functions.         #
   #       v2.0 (04-2002)                             #
   #       v2.01(02-2003)                             #
   ####################################################

   This file is part of NetMeter.

   NetMeter is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   NetMeter is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with NetMeter; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 */

#include <sys/time.h>
#include <time.h>

#include "flows.h"

#ifndef __COMMON_H__
#define __COMMON_H__

// Error defines
#define OK        0
#define MYEINVAL  1
#define ENOFD     2
#define EEOF      3
#define EBIGLINE  4
#define ENOINT    5

#define MAX_LINE 250
#define C_SEPARATOR "_ +[]{}()=!\"$%&/?\t\n"

#define MAX_FLOWS 20
#define MAX_PKT   70000

#define IPv4 1
#define IPv6 2

#define TSSIZE 16

#define MASK(offset_) 1 << offset_

#ifndef TRUE
	#define TRUE 1
#endif

#ifndef FALSE
	#define FALSE 0
#endif

/* Function to read a double from the input stream */
int read_double(int fd, double *fl, int is_verbose);
/* Function to read a line from the input stream, max MAX_LINE chars*/
int read_line(int fd, char *fl, int is_verbose);

int to_integer(char c);
int to_string(long long number, unsigned int size, char *result);
int to_msec(struct timeval *time1, double *result);

int is_separator(char c);
int is_number(char c);
int is_hex(char c);

int getInteger(char *data);
int getIP(char *data, p_address addr);
int getTime(char *data, struct timeval *timestamp);

int time_diff(struct timeval *time1, struct timeval *time2, struct timeval *result);
int time_sum(struct timeval *time1, struct timeval *time2, struct timeval *result);
int time_div(struct timeval *time1, long int divisor, struct timeval *result);
int ts_is_bigger(struct timeval *time1, struct timeval *time2);
int ts_is_smaller(struct timeval *time1, struct timeval *time2);
int ts_is_bigger_or_equal(struct timeval *time1, struct timeval *time2);
int ts_is_smaller_or_equal(struct timeval *time1, struct timeval *time2);
int ts2char(struct timeval *time1, char result[TSSIZE]);
int ts2double(struct timeval *time1, double *result);

#endif
