/* ####################################################
   # Copyright 2002, 2003 Ren Serral,                #
   #                                                  #
   # flows.c - Flow Management.                       #
   #       v2.0 (04-2002)                             #
   #       v2.01(02-2003)                             #
   ####################################################

   This file is part of NetMeter.

   NetMeter is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   NetMeter is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with NetMeter; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 */

#include <stdio.h>

#include "flows.h"
#include "common.h"

p_flist_head new_flist(p_flow item) {
	p_flist_head list;
	p_flist first;
	p_flow new_item;

	list = (p_flist_head)malloc(sizeof(flist_head));
	list->elements = 0;
	list->max_seq = 0;
	if (!item) {
#ifdef DEBUG
		fprintf(stderr,"Trying to create an empty list\n");
#endif
		list->head = 0;
		list->tail = 0;
		return list;
	}
	first = (p_flist)malloc(sizeof(flist));
	new_item = (p_flow)malloc(sizeof(flow));
	if (!new_item || !list || !first) {
#ifdef DEBUG
		fprintf(stderr,"I don't have enought memory, sorry!\n");
#endif
		return NULL;
	}
	list->head = first;
	list->tail = first;
	list->actual = first;
	memcpy(new_item, item, sizeof(flow));
	first->item = new_item;
	first->next = 0;
	first->prev = 0;
	list->elements++;
	return list;
}

p_flist_head copy_flow(p_flist_head list, int my_flow) {
	p_flist_head new_list;
	p_flow tmp;
	if (!list) {
#ifdef DEBUG
		fprintf(stderr,"Trying to copy an empty list\n");
#endif
		return NULL;
	}
	new_list = (p_flist_head)malloc(sizeof(flist_head));
	new_list->elements = 0;
	new_list->max_seq = 0;
	new_list->head = 0;
	new_list->tail = 0;
	if (!new_list) {
#ifdef DEBUG
		fprintf(stderr,"I don't have enought memory, sorry!\n");
#endif
		return NULL;
	}
	tmp = go_first(list);
	do {
		if (tmp->flow_id == my_flow) {
			add_item(new_list, tmp);
		}
		tmp = go_next(list);
	} while (tmp);
	new_list->actual = new_list->head;
	return new_list;
}

int del_list(p_flist_head list) {
	p_flist tmp, next;

	tmp = list->head;
	while (tmp != list->tail) {
		free (tmp->item);
		next = tmp->next;
		free(tmp);
		tmp = next;
	}
	free(tmp->item);
	free(tmp);

	free(list);
	return OK;
}

int add_item(p_flist_head list, p_flow item) {
	p_flist new_item;
	if (!list) {
#ifdef DEBUG
		fprintf(stderr,"Give me a list!\n");
#endif
		return -MYEINVAL;
	}
	if (!item) {
#ifdef DEBUG
		fprintf(stderr,"Inserting NULL item?, NO, i don't\n");
#endif
		return -MYEINVAL;
	}

	new_item = (p_flist)malloc(sizeof(flist));
	if (!new_item) {
#ifdef DEBUG
		fprintf(stderr,"I don't have enought memory, sorry!\n");
#endif
		return -MYEINVAL;
	}
	new_item->item = (p_flow)malloc(sizeof(flow));
	memcpy(new_item->item, item, sizeof(flow));

	new_item->next = 0;
	if (list->tail) {
		new_item->prev = list->tail;
		list->tail->next = new_item;
	} else {	// New List
		new_item->prev = 0;
		list->head = new_item;
	}
	list->tail = new_item;
	list->elements++;
	if (item->seq_num > list->max_seq)
		list->max_seq = item->seq_num;
	return OK;
}

int add_item_sorted_inc(p_flist_head list, p_flow item) {
	struct timeval ts_new, ts_act;
	p_flist new_item;
	p_flist aux, a_prev;
	if (!list) {
#ifdef DEBUG
		fprintf(stderr,"Give me a list!\n");
#endif
		return -MYEINVAL;
	}
	if (!item) {
#ifdef DEBUG
		fprintf(stderr,"Inserting NULL item?, NO, i don't\n");
#endif
		return -MYEINVAL;
	}

	new_item = (p_flist)malloc(sizeof(flist));
	if (!new_item) {
#ifdef DEBUG
		fprintf(stderr,"I don't have enought memory, sorry!\n");
#endif
		return -MYEINVAL;
	}
	new_item->item = (p_flow)malloc(sizeof(flow));
	memcpy(new_item->item, item, sizeof(flow));

	new_item->next = 0;
	if (list->head) {
		aux = list->head;
		time_diff(&new_item->item->rx_time, &new_item->item->tx_time, &ts_new);
		do {
			time_diff(&aux->item->rx_time, &aux->item->tx_time, &ts_act);
			a_prev = aux;
//			if (aux->next && ts_is_bigger(&ts_act, &ts_new))
			aux = aux->next;
		} while (aux && ts_is_bigger(&ts_act, &ts_new));
		aux = a_prev;
		if (!aux->next && !ts_is_bigger(&ts_act, &ts_new)) {
			new_item->prev = list->tail;
			list->tail->next = new_item;
			new_item->prev = list->tail;
			list->tail = new_item->prev;
		} else {
			if (list->head == list->tail)
				list->head = new_item;
			else
				aux->prev->next = new_item;
			new_item->prev = aux->prev;
			new_item->next = aux;
			aux->prev = new_item;
		}
	} else {
		new_item->prev = 0;
		list->head = new_item;
		list->tail = new_item;
	}
	list->elements++;
	if (item->seq_num > list->max_seq)
		list->max_seq = item->seq_num;
	return OK;
}

p_flow go(p_flist_head list, p_flow item) {
	if (!list) {
#ifdef DEBUG
		fprintf(stderr,"Can't search an item on a NULL list!\n");
#endif
		return NULL;
	}
	if (!item) {
#ifdef DEBUG
		fprintf(stderr,"Item not found (is NULL\n)");
#endif
		return NULL;
	}
	list->actual = list->head;
	while (list && list->actual->item != item)
		list->actual = list->actual->next;
	if (list->actual->item != item)
		return NULL;
	return list->actual->item;
}
p_flow go_first(p_flist_head list) {
	if (!list) {
#ifdef DEBUG
		fprintf(stderr,"Can't go first item of a NULL list!\n");
#endif
		return NULL;
	}
	list->actual = list->head;
	if (list->actual)
		return list->actual->item;
	return NULL;
}

p_flow go_last(p_flist_head list) {
	if (!list) {
#ifdef DEBUG
		fprintf(stderr,"Can't go first item of a NULL list!\n");
#endif
		return NULL;
	}
	list->actual = list->tail;
	if (list->actual)
		return list->actual->item;
	return NULL;
}

p_flow go_next(p_flist_head list) {
	if (!list) {
#ifdef DEBUG
		fprintf(stderr,"Can't go first item of a NULL list!\n");
#endif
		return NULL;
	}
	if (!list->actual)
		list->actual = list->head;
	else
		list->actual = list->actual->next;
	if (!list->actual) {
		list->actual = list->tail;
		return NULL;
	}
	return list->actual->item;
}

p_flow go_prev(p_flist_head list) {
	if (!list) {
#ifdef DEBUG
		fprintf(stderr,"Can't go first item of a NULL list!\n");
#endif
		return NULL;
	}
	if (!list->actual)
		list->actual = list->tail;
	else
		list->actual = list->actual->prev;
	if (!list->actual) {
		list->actual = list->head;
		return NULL;
	}
	return list->actual->item;
}

p_flow give_flow(p_flist_head list) {
	if (!list) {
#ifdef DEBUG
		fprintf(stderr,"Can't go first item of a NULL list!\n");
#endif
		return NULL;
	}
	if (list->actual)
		return list->actual->item;
	else
		return NULL;
}

p_flow make_flow(char *data) {
	p_flow item;
	char *tmp;

	item = (p_flow)malloc(sizeof(flow));
	if (!data) {
#ifdef DEBUG
		fprintf(stderr, "NULL data is no FLOW\n");
#endif
		free(item);
		return NULL;
	}
	// Flow to be treated
	tmp = strstr(data, FLOW_STR);
	if (!tmp) {
#ifdef DEBUG
		fprintf(stderr, "Invalid string format in Flow identification\n");
#endif
		free(item);
		return NULL;
	}
	tmp += strlen(FLOW_STR);
	item->flow_id = getInteger(tmp);

	// Sequence number
	tmp = strstr(tmp, SEQ_STR);
	if (!tmp) {
#ifdef DEBUG
		fprintf(stderr, "Invalid string format in Sequence identification\n");
#endif
		free(item);
		return NULL;
	}
	tmp += strlen(SEQ_STR);
	item->seq_num = getInteger(tmp);

	// Source IP
	tmp = strstr(tmp, SRC_STR);
	if (!tmp) {
#ifdef DEBUG
		fprintf(stderr, "Invalid string format in SOURCE identification\n");
#endif
		free(item);
		return NULL;
	}
	tmp += strlen(SRC_STR);
	item->protocol = getIP(tmp, &(item->s_addr));

	// Destination IP
	tmp = strstr(tmp, DEST_STR);
	if (!tmp) {
#ifdef DEBUG
		fprintf(stderr, "Invalid string format in DESTINATION identification\n");
#endif
		free(item);
		return NULL;
	}
	tmp += strlen(DEST_STR);
	getIP(tmp, &(item->d_addr));

	// TxTime
	tmp = strstr(tmp, TX_STR);
	if (!tmp) {
#ifdef DEBUG
		fprintf(stderr, "Invalid string format in TxTime identification\n");
#endif
		free(item);
		return NULL;
	}
	tmp += strlen(TX_STR);
	getTime(tmp, &(item->tx_time));

	// RxTime
	tmp = strstr(tmp, RX_STR);
	if (!tmp) {
#ifdef DEBUG
		fprintf(stderr, "Invalid string format in RxTime identification\n");
#endif
		free(item);
		return NULL;
	}
	tmp += strlen(RX_STR);
	getTime(tmp, &(item->rx_time));

	// Packet Size
	tmp = strstr(tmp, SIZE_STR);
	if (!tmp) {
#ifdef DEBUG
		fprintf(stderr, "Invalid string format in Size identification\n");
#endif
		free(item);
		return NULL;
	}
	tmp += strlen(SIZE_STR);
	item->size = getInteger(tmp);
	return item;
}

p_flist_head make_flow_list(int fd, int is_verbose) {
	int ok;
	char line[MAX_LINE];
	long int my_max_seq=-1;
	struct timeval acc_delay, ts_tmp;
	p_flist_head hList;
	p_flow myFlow;

	hList = init_list();
	if (!hList)
		return NULL;
	bzero(&acc_delay, sizeof(struct timeval));
	do {
		bzero(line,MAX_LINE);
		ok = read_line(fd, line, is_verbose);
		if (ok >= 0) {
			myFlow = make_flow(line);
			if (myFlow) {
				add_item(hList,myFlow);
				// Update statistics information
				if (time_diff(&(myFlow->rx_time), &(myFlow->tx_time), &ts_tmp)) {
					del_list(hList);
					return NULL;
				}
				time_sum(&acc_delay, &ts_tmp, &acc_delay);
				if (ts_is_bigger(&ts_tmp, &(hList->max_delay)))
					memcpy(&(hList->max_delay), &ts_tmp, sizeof(struct timeval));
				if (ts_is_smaller(&ts_tmp, &(hList->min_delay)))
					memcpy(&(hList->min_delay), &ts_tmp, sizeof(struct timeval));
				hList->arrived_packets++;
				// Duplicated packets are not taken into account!
				if (my_max_seq < (myFlow->seq_num + 1)) {
					hList->packet_losses += myFlow->seq_num - my_max_seq - 1;
					my_max_seq = myFlow->seq_num;
				} else if (my_max_seq > myFlow->seq_num) {
					hList->packet_losses--;
					hList->outoforder_packets++;
				}
			}
		}
	} while(ok > 0);
	time_div(&acc_delay, hList->arrived_packets, &(hList->mean_delay));
	hList->test_packets = hList->arrived_packets + hList->packet_losses;
	return hList;
}

int write_back(p_flist_head list) {
	int pos, sLen;
	char line[MAX_LINE], tmp[MAX_LINE];
	p_flow hlNode;

//#define DEST_STR "Dest>"
//#define TX_STR   "TxTime>"
//#define RX_STR   "RxTime>"
//#define SIZE_STR "Size>"
//int protocol;
//address s_addr;
//address d_addr;
//struct timeval tx_time, rx_time;
//int size;

	if (!list) {
		return -MYEINVAL;
	}
	hlNode = go_first(list);
	do {
		sLen = strlen(FLOW_STR);
		pos = sLen;
		strncpy(line, FLOW_STR, pos);
		to_string(hlNode->flow_id, 4, &(line[pos]));
		pos += 4;
		line[pos++]=' ';
		sLen = strlen(SEQ_STR);
		pos += sLen;
		strncpy(line, SEQ_STR, sLen);
		to_string(hlNode->seq_num, 6, &(line[pos]));
		pos += 6;
		line[pos++]=' ';
		sLen = strlen(SRC_STR);
		pos += sLen;
		if (hlNode->protocol == IPv4) {
			*tmp = inet_ntoa(hlNode->s_addr.s_address);
		} else {
			*tmp = inet_ntoa(hlNode->s_addr.s6_address);
		}
		sLen = strlen(tmp);
		pos += sLen;
		strncpy(&(line[pos]), tmp, sLen);
		line[pos++]=' ';
		sLen = strlen(DEST_STR);
		pos += sLen;
		if (hlNode->protocol == IPv4) {
			*tmp = inet_ntoa(hlNode->d_addr.s_address);
		} else {
			*tmp = inet_ntoa(hlNode->d_addr.s6_address);
		}
		sLen = strlen(tmp);
		pos += sLen;
		strncpy(&(line[pos]), tmp, sLen);
		line[pos++]=' ';
// m'he quedat al dest!

	} while (hlNode = go_next(list));


}

p_flist_head init_list() {
	p_flist_head hList;
	hList = new_flist(NULL);
	if (!hList)
		return NULL;
	hList->min_delay.tv_sec = 1000;
	hList->min_delay.tv_usec = 999999;
	hList->max_delay.tv_sec = 0;
	hList->max_delay.tv_usec = 0;
	hList->arrived_packets = 0;
	hList->test_packets = 0;
	hList->outoforder_packets = 0;
	return hList;
}
