/* ####################################################
   # Copyright 2002, 2003 Ren Serral,                #
   #                                                  #
   # mos.h - VoIP MOS calculation as                  #
   #         stated onsaid on ITU-T G.107             #
   #       v2.0 (04-2002)                             #
   #       v2.01(02-2003)                             #
   ####################################################

   This file is part of NetMeter.

   NetMeter is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   NetMeter is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with NetMeter; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 */

struct parameters {
	double SLR;
	double RLR;
	double STMRs;
	double Ds;
	double STMR;
	double Dr;
	double LSTRr;
	double TELR;
	double T;
	double WEPL;
	double Tr;
	double Ta;
	double Ie;
	double Bpl;
	double Ppl;
	double BurstR;
	double A;
	double Nc;
	double Ps;
	double Pr;
	double qdu;
	double Nfor;
};

/* Default parameters for the MOS calculation
 */
void init(struct parameters *params);

/* Formula 3 to 7 of the ITU-T G.107 Specs
 */
double Nt(struct parameters *params);
/* Formula 2 of the ITU-T G.107 Specs
 */
double Ro(struct parameters *params);
/* Formula 9 and 10 of the ITU-T G.107 Specs
 */
double Iolr(struct parameters *params);
/* Formula 11 to 12 of the ITU-T G.107 Specs
 */
double Ist(struct parameters *params);
/* Formula 13 to 17 of the ITU-T G.107 Specs
 */
double Iq(struct parameters *params);
/* Formula 8 of the ITU-T G.107 Specs
 */
double Is(struct parameters *params);
/* Formula 22 and 23 of the ITU-T G.107 Specs
 */
double TERV(struct parameters *params);
/* Formula 19, 21 and 24 of the ITU-T G.107 Specs
 */
double Idte(struct parameters *params);
/* Formula 25 and 26 of the ITU-T G.107 Specs
 */
double Idle(struct parameters *params);
/* Formula 27 and 28 of the ITU-T G.107 Specs
 */
double Idd(struct parameters *params);
/* Formula Id ITU-T G.107 Specs
 */
double Id(struct parameters *params);
/* Formula 29, packet loss inclusion of the ITU-T G.107 Specs
 */
double Ieef(struct parameters *params);
/* Formula 1, R computation of ITU-T G.107 Specs
 */
double R(struct parameters *params);
