/* ####################################################
   # Copyright 2002, 2003 Ren Serral,                #
   #                                                  #
   # loss_interval.c - Recovery time calculator.      #
   #       v2.0 (04-2002)                             #
   #       v2.01(02-2003)                             #
   ####################################################

   This file is part of NetMeter.

   NetMeter is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   NetMeter is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with NetMeter; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <time.h>

#include <math.h>

#include "lib/common.h"
#include "lib/flows.h"
#include "lib/arbolAVLDouble.h"
//#include "lib/sequence.h"

#define CHUNK 5000
#define DEF_OUTLIER 0.99

/* Procedure for displaying usage information */
void usage();
/* Temp function for printing the status */
void printValue(double *value);

/* Global Variable declaration */
int sh_max, sh_min, sh_avg, sh_values;
int is_verbose;

int main(int argc, char **argv) {
	int sel_opt, n_elements, ok;
	int fd_read1, fd_write; // File Handlers

	int is_verbose = 0, has_input = 0, has_output = 0,
		has_force = 0; // Boolean switches
	char *out_file; // Output file
	char line[MAX_LINE];

	int i, tmp_seq, n_seq;
	char *sequence;
	struct timeval time1;
#ifdef DEBUG
	struct timeval t_start, t_end, t_res;
	long int t_tmp;
#endif
	p_flist_head hList;
	p_flow hLnode;
	struct timeval ts_final;

	ArbolDouble avlList = NULL;
	double result;
	float f_percent;

	// Inicialitzations
	fd_read1  = STDIN_FILENO;
	fd_write  = STDOUT_FILENO;
	bzero (line, MAX_LINE);

	f_percent = DEF_OUTLIER;
	// Main option parsing loop
	while(1) {
		sel_opt = getopt(argc, argv, "i:o:vfhp:");
		if (sel_opt == -1)
			break;
		switch (sel_opt) {
		case 'i':
			if (has_input) {
				fprintf(stderr,"Ingoring more input files\n");
				break;
			}
			fd_read1 = open(optarg,O_RDONLY);
			if (fd_read1 < 0) {
				perror("Error opening input file 1");
				exit(-1);
			}
			has_input = 1;
		break;
		case 'o':
			if (has_output)
				break;
			out_file = malloc(strlen(optarg)+1);
			strncpy(out_file,optarg,strlen(optarg));
			has_output = 1;
		break;
		case 'v':
			is_verbose = 1;
		break;
		case 'f':
			has_force = 1;
		break;
		case 'p':
			f_percent = atof(optarg);
			if (f_percent == 0 || f_percent > 1) {
				fprintf(stderr, "When specifying the -p flag the value should be between 0 and 1 indicating the threshold to be considered as an outlier: i.e. 0.99\n");
				exit(-3);
			}
			break;
		case 'h':
			usage();
			if (fd_read1 != STDIN_FILENO)
				close(fd_read1);
			exit(0);
		break;
		default:
			usage();
			exit(-2);
		break;
		} // End Switch sel_opt
	} // End While (option parsing)
	if (has_output) {
		if (!has_force) {
			fd_write = open(out_file, O_RDONLY);
			if (fd_write > 0) {
				close(fd_write);
				close(fd_read1);
				fprintf(stderr,"Error: Output file %s exists, use -f to override\n", out_file);
				exit(-1);
			}
		}
		fd_write = open(out_file, O_RDWR | O_CREAT, 0666);
		if (fd_write < 0) {
			perror("Error creating output file");
			exit(-1);
		}
	} // End if has_output

	i = 0;
#ifdef DEBUG
	gettimeofday(&t_start, NULL);
#endif
	hList = make_flow_list(fd_read1, is_verbose);
#ifdef DEBUG
	gettimeofday(&t_end, NULL);
	time_diff(&t_end, &t_start, &t_res);
	fprintf(stderr, "Flow list creation time: %d sec, %d usec\n", t_res.tv_sec, t_res.tv_usec);
	if (hList) {
		t_tmp = (t_res.tv_sec * 1000000l + t_res.tv_usec) / hList->elements;
		fprintf(stderr, "Total list elements: %d, %d sec and %d usec of computing time per packet\n",
			hList->elements, t_tmp / 1000000l, t_tmp % 1000000l);
		fprintf(stderr, "Maximum sequence number: %d\n", hList->max_seq);
	}
#endif
	/* Starting to calculate delays, and inserting them on the AVL */
/*	if (hLnode = go_first(hList)) {
		do {
			time_diff(&(hLnode->rx_time), &(hLnode->tx_time), &ts_final);
			to_msec(&ts_final, &result);
			InsertarDouble(&avlList, result);
		} while (hLnode = go_next(hList));
		InOrdenDouble(avlList, &printValue);
	}
*/
struct timeval ts_tdiff;
double delay_diff, d_base;
int mem_to_go, i_base;
long int acum_elems, wanted;
typedef struct outlier__ {
	int num_elems;
	p_flist_head packets;
} outlier, *p_outlier;
p_outlier out_list;
	if (hLnode = go_first(hList)) {
		time_diff(&hList->mean_delay, &hList->min_delay, &ts_tdiff);
		to_msec(&ts_tdiff, &delay_diff);
		to_msec(&hList->min_delay, &d_base);
		i_base = d_base;
		acum_elems = 0;
		mem_to_go = delay_diff * 3 + 1;
		out_list = malloc(mem_to_go * sizeof(outlier));
		bzero(out_list, mem_to_go * sizeof(outlier));
		do {
			time_diff(&(hLnode->rx_time), &(hLnode->tx_time), &ts_final);
			to_msec(&ts_final, &result);
			result = result - i_base;
			if ((int)result >= (mem_to_go - 1)) {
				result = mem_to_go - 1;
				out_list[(int)result].num_elems++;
				if (!out_list[(int)result].packets)
					out_list[(int)result].packets = init_list();
				add_item_sorted_inc(out_list[(int)result].packets, hLnode);
			} else {
				out_list[(int)result].num_elems++;
				acum_elems++;
			}
//printf("%d, %d\n", (int)result, out_list[(int)result].num_elems);
		} while (hLnode = go_next(hList));
	}
	wanted = floor(hList->arrived_packets * f_percent);

	if (wanted < acum_elems) {
		fprintf(stderr, "Sorry, too many outliers, this program is not able to handle them for now. Sure next version will\n");
	} else {
		hLnode = go_first(out_list[mem_to_go-1].packets);
		for (acum_elems; acum_elems < wanted; acum_elems++)
			hLnode = go_next(out_list[mem_to_go-1].packets);
		do {
			printf("%d is an outlier\n", hLnode->seq_num);
		} while (hLnode = go_next(out_list[mem_to_go-1].packets));
	}
	
	close(fd_read1);
	close(fd_write);
	del_list(hList);
	return 0;
}

void printValue(double *value) {
	printf("%f\n", *value);
}

/* Procedure for displaying usage information */
void usage() {
	printf("Usage: ./loss_interval -i <input_file> -o <output_file> [-f] [-v]\n");
	printf("	-i   input_file, if is omitted stdin will be used\n");
	printf("	-o   output_file, if is omitted stdout will be used\n");
	printf("	-f   for overwriting destination file\n");
	printf("	-v   Be verbose\n");
}
