/* ####################################################
   # Copyright 2002, 2003 Ren Serral,                #
   #                                                  #
   # loss_interval.c - Recovery time calculator.      #
   #       v2.0 (04-2002)                             #
   #       v2.01(02-2003)                             #
   ####################################################

   This file is part of NetMeter.

   NetMeter is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   NetMeter is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with NetMeter; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <time.h>

#include <math.h>

#include "lib/common.h"
#include "lib/flows.h"

#define CHUNK 5000

/* Procedure for displaying usage information */
void usage();

/* Global Variable declaration */
int sh_max, sh_min, sh_avg, sh_values;
int is_verbose;

int main(int argc, char **argv) {
	int sel_opt, n_elements, ok;
	int fd_read1, fd_write; // File Handlers

	int is_verbose = 0, has_input = 0, has_output = 0,
		has_force = 0; // Boolean switches
	char *out_file; // Output file
	char line[MAX_LINE];

	int i, tmp_seq, n_seq;
	long long int current;
	struct timeval time1;
#ifdef DEBUG
	struct timeval t_start, t_end, t_res;
	long int t_tmp;
#endif
	p_flist_head hList;
	p_flow myFlow;

	// Inicialitzations
	fd_read1  = STDIN_FILENO;
	fd_write  = STDOUT_FILENO;
	bzero (line, MAX_LINE);

	// Main option parsing loop
	while(1) {
		sel_opt = getopt(argc, argv, "i:o:vfh");
		if (sel_opt == -1)
			break;
		switch (sel_opt) {
		case 'i':
			if (has_input) {
				fprintf(stderr,"Ingoring more input files\n");
				break;
			}
			fd_read1 = open(optarg,O_RDONLY);
			if (fd_read1 < 0) {
				perror("Error opening input file 1");
				exit(-1);
			}
			has_input = 1;
		break;
		case 'o':
			if (has_output)
				break;
			out_file = malloc(strlen(optarg+1));
			strncpy(out_file,optarg,strlen(optarg));
			has_output = 1;
		break;
		case 'v':
			is_verbose = 1;
		break;
		case 'f':
			has_force = 1;
		break;
		case 'h':
			usage();
			if (fd_read1 != STDIN_FILENO)
				close(fd_read1);
			exit(0);
		break;
		default:
			usage();
			exit(-2);
		break;
		} // End Switch sel_opt
	} // End While (option parsing)
	if (has_output) {
		if (!has_force) {
			fd_write = open(out_file, O_RDONLY);
			if (fd_write > 0) {
				close(fd_write);
				close(fd_read1);
				fprintf(stderr,"Error: Output file %s exists, use -f to override\n", out_file);
				exit(-1);
			}
		}
		fd_write = open(out_file, O_RDWR | O_CREAT, 0666);
		if (fd_write < 0) {
			perror("Error creating output file");
			exit(-1);
		}
	} // End if has_output

	i = 0;
#ifdef DEBUG
	gettimeofday(&t_start, NULL);
#endif
	hList = make_flow_list(fd_read1, is_verbose);
#ifdef DEBUG
	gettimeofday(&t_end, NULL);
	time_diff(&t_end, &t_start, &t_res);
	fprintf(stderr, "Flow list creation time: %d sec, %d usec\n", t_res.tv_sec, t_res.tv_usec);
	if (hList) {
		t_tmp = (t_res.tv_sec * 1000000l + t_res.tv_usec) / hList->elements;
		fprintf(stderr, "Total list elements: %d, %d sec and %d usec of computing time per packet\n",
			hList->elements, t_tmp / 1000000l, t_tmp % 1000000l);
		fprintf(stderr, "Maximum sequence number: %d\n", hList->max_seq);
	}
#endif
	myFlow = go_first(hList);
	current = 0;
	tmp_seq = 0;
	do {
		n_seq = myFlow->seq_num;
		if (tmp_seq > n_seq) { // Just found OoO packet
			if (is_verbose)
				fprintf(stdout, "\tPacket out of order at %d\n", n_seq);
			current++;
		}
		else {
			tmp_seq = n_seq;
		}
	} while (myFlow = go_next(hList));

	if (fd_write > STDOUT_FILENO) {
	}
	else {
		fprintf(stdout, "\tTotal: %d Out of Order packets\n", current);
	}
	close(fd_read1);
	close(fd_write);
	del_list(hList);
	return 0;
}

/* Procedure for displaying usage information */
void usage() {
	printf("Usage: ./outoforder -i <input_file> -o <output_file> [-f] [-v]\n");
	printf("	-i   input_file, if is omitted stdin will be used\n");
	printf("	-o   output_file, if is omitted stdout will be used\n");
	printf("	-f   for overwriting destination file\n");
	printf("	-v   Be verbose\n");
}
