###############################################
# Copyright 2002, 2003 Ren Serral,           #
#                                             #
# delaym4b.tcl - Multicast Delay IPv4 Manager #
#       v1.0 (10-2002)                        #
###############################################

#   This file is part of NetMeter.
#
#   NetMeter is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   NetMeter is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with NetMeter; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

###################################################################
#------------------------------------------------------------------
#                  RUN Multicast DELAY TEST
#------------------------------------------------------------------
###################################################################
#generate the Mgen and Drec Script
#send the Mgen and Drec Script
#execute remote Mgen Drec
#savve the result in local machine
proc RunMDelayTest {} \
{
   global sender receiver host multicast multicastEnd
   # dlyM: general options for all the defined flows
   global dlyM
   # flowM: specific options for the defined flowM(param,flowM_number)
   global flowM
   global mod
   global customParamsD customEndD
   global runB logDelayLB
   global mID dID
   global stopDrecCmd stopMgenCmd
   global success
   global getDrecLog
   global decodepath4
   global SGN
   global _sshCMD

#if You don't want to launch the Test.... only process .DLY :(
#   global fPath
#   set fPath "/home/roberto/netmeter/data/X.dlyM"
#   ProcessMData
#   return

   #check if there is at least one flowM defined:
   if !$flowM(number) \
   {
      #0 flows
     return
   }

   #set well done test flag to 0:
   set success 0

   #clear status console (message window)
   $runB config -text "Wait..."
   $logDelayLB delete 0 end
   update idletasks

   #find sender and receiver:
   if $sender(local,ipv4,multicast) {
      set senderHost $host(address,ipv4,multicast)
      set mgenCmd $dlyM(mgenPath,ipv4)
      set gscriptCmd "cat > /tmp/gscript"
      set stopMgenCmd "killall $SGN mgen"
   } else {
      set senderHost $sender(address,ipv4,multicast)
      set mgenCmd     "$_sshCMD -l $sender(login,ipv4,multicast) $senderHost $dlyM(mgenPath,ipv4) "
      set gscriptCmd  "$_sshCMD -l $sender(login,ipv4,multicast) $senderHost {cat > /tmp/gscript}"
      set stopMgenCmd "$_sshCMD -l $sender(login,ipv4,multicast) $senderHost killall $SGN mgen"
   }

   ##################################################################
   # NOTE over MGEN-DREC SIGINT: 
   # If you don't use SIGINT drec don't finish to write the Received
   # buffer missing the last 5% of output data!!!!
   # also MGEN don't put statistics information :(
   ##################################################################

   ##################################################################
   ### Begin preliminar script inicialitzation
   ###   here we generate the script that'll be sent to the 
   ###   multicast group hosts --> that is all unicast addresses
   ###   given in the hosts listbox on page 1.
   ##################################################################
   ##construct lists with script commands
   #put a message on window
   ##################################################################
   ### End preliminar script inicialitzation
   ##################################################################
   
   ##################################################################
   ### Loop for all hosts to send the execution script
   ### TODO: Show a dialog with the stats of the connections
   for {set actual 0} {$actual <= $multicastEnd(ipv4)} {incr actual} {
	if $multicast(local,ipv4,$actual) {
		set receiverHost $host(address,ipv4,multicast)
		set drecCmd $dlyM(drecPath,ipv4)
		set dscriptCmd "cat > /tmp/dscript"
		set stopDrecCmd "killall $SGN drec"
		#OLD MGEN4 version= set getDrecLog  "cat /tmp/logDrec"
		set getDrecLog  "cat /tmp/output.txt"
		#Waiting the DECODE set getDrecLog  "$decodepath4 /tmp/output.bin | cat"
	} else {
		set receiverHost $multicast(address,ipv4,$actual)
		# If the login name is empty not append the -l parameter!!!
		if {[string length $multicast(login,ipv4,$actual)] > 0} {
			set realSSH "$_sshCMD -l $multicast(login,ipv4,$actual)"
		} else {
			set realSSH "$_sshCMD"
		}
		set drecCmd     "$realSSH $receiverHost $dlyM(drecPath,ipv4) "
		set dscriptCmd  "$realSSH $receiverHost {cat >/tmp/dscript}"
		#OLD MGEN4 version= set getDrecLog  "$_sshCMD -l $receiver(login,ipv4)  $receiverHost cat /tmp/logDrec"
		set getDrecLog  "$realSSH $receiverHost cat /tmp/output.txt"
		#Waiting the DECODE set getDrecLog  "$_sshCMD -l $receiver(login,ipv4)  $receiverHost $decodepath4 /tmp/output.bin | cat"
		set stopDrecCmd "$realSSH $receiverHost killall $SGN drec"
	}
	### Here we already have all actual host data so we can manage sending the script there.

	$logDelayLB insert end "Building scripts..."
	$logDelayLB see end
	update idletasks

	lappend gscriptList "# mgen launching script"
	lappend dscriptList "# drec launching script"

	#if we have a start command
	if $dlyM(startBool) {
		#[trim]->kill space & tab chars
		#if ($dlyM(start)!="") then .....
		if {[string trim $dlyM(start)] != ""} {
			lappend gscriptList "START $dlyM(start)"
			lappend dscriptList "START $dlyM(start)"
		}
	}

	#if we have a srcPort command for mgen
	#the port for the source flowM is ONE
	#for drec we have to set the ports for all the flows
	if $dlyM(srcPortBool) {
		if {[string trim $dlyM(srcPort)] != ""} {
		lappend gscriptList "PORT $dlyM(srcPort)"
		}
	}

	#interfaces:
	#mgen interface
	if $sender(local,ipv4,multicast) {
		set sInterface $host(interface,ipv4,multicast)
	} else {
		set sInterface $sender(interface,ipv4,multicast)
	}
	if {[string trim $sInterface] != ""} {
		lappend mgenCmd "-i $sInterface"
	}

	#drec interface
	if $receiver(local,ipv4) {
			set rInterface $host(interface,ipv4,multicast)
	} else {
			set rInterface $receiver(interface,ipv4)
	}

	if {[string trim $rInterface] != ""} {
			lappend dscriptList "INTERFACE $rInterface"
	}

	#more flowM with different destination port
	#ports listened by drec:
	for {set i 1} {$i <= $flowM(number)} {incr i} {
			lappend portList $flowM(port,$i)
	}

	#this loop will run at least once,
	#so portList will certainly exist
	lappend dscriptList "PORT [join $portList {,}]"

	#Test Watch for script starts time and ports
	#tk_messageBox -message "DREC Script List:\n  $dscriptList"
	#tk_messageBox -message "MGEN Script List:\n  $gscriptList"

	#deal with flows:
	#construct two temporary lists with commands that
	#will be sorted chronologically afterwards (see the start time)
	lappend drecTempList ""
	for {set i 1} {$i <= $flowM(number)} {incr i} \
	{
		set unresv 0
		#temporary line
		set tempLine "$flowM(start,$i) $i ON $receiverHost:$flowM(port,$i) \
			$flowM(pattern,$i) $flowM(rate,$i) $flowM(size,$i)"
		#the flowM(i) use RSVP parameters ?
		if $flowM(rsvpBool,$i) {
			if {[string trim $flowM(rsvp,$i)] != ""} \
			{
				append tempLine " RSVP $flowM(rsvp,$i)"
				#RESV:
				lappend drecTempList "$flowM(start,$i) RESV $receiverHost:$flowM(port,$i) \
				$flowM(style,$i) $flowM(resv,$i)"
				lappend drecTempList "$flowM(stop,$i) UNRESV $receiverHost:$flowM(port,$i)"
				set unresv 1
			}
		}
		#the flowM(i) use TYPE of SERVICE parameters ?
		if $flowM(tosBool,$i) {
			if {[string trim $flowM(tos,$i)] != ""} \
			{
				append tempLine " TOS $flowM(tos,$i)"
			}
		}
		lappend mgenTempList $tempLine

		#note: a generic flux(i) is associeted a $mod(number,$i) modify flows
		#deal with MOD data
		for {set j 1} {$j <= $mod(number,$i)} {incr j} \
		{
			set tempLine "$mod(start,$j,$i) $i MOD $receiverHost:$flowM(port,$i) \
				$mod(pattern,$j,$i) $mod(rate,$j,$i) $mod(size,$j,$i)"
			#the mod(i) use RSVP parameters ?
			if $mod(rsvpBool,$j,$i) {
				if {[string trim $mod(rsvp,$j,$i)] != ""}\
				{
				append tempLine " RSVP $mod(rsvp,$j,$i)"
				lappend drecTempList "$mod(start,$j,$i) RESV $receiverHost:$flowM(port,$i) \
					$mod(style,$j,$i) $mod(resv,$j,$i)"
				if {!$unresv} {
					lappend drecTempList "$flowM(stop,$i) UNRESV $receiverHost:$flowM(port,$i)"
					set unresv 1
				}
				}
			}
			#the mod(i) use TYPE of SERVICE parameters ?
			if $mod(tosBool,$j,$i) {
			if {[string trim $mod(tos,$j,$i)] != ""} {
				append tempLine " TOS $mod(tos,$j,$i)"
			}
			}
			lappend mgenTempList $tempLine
		}
		lappend mgenTempList "$flowM(stop,$i) $i OFF"
	}
	#sort mgenTempList and drecTempList
	if [catch {lsort -index 0 -integer $mgenTempList} mgenTempList] \
	{
		tk_messageBox -message "All flows start-stop values\nmust be integers"
		return
	}

	#all time values are integers
	#no need to catch next command (uses the same start-stop values)
	if {[string trim $drecTempList] != ""} \
	{
		set drecTempList [lreplace $drecTempList 0 0]
		set drecTempList [lsort -index 0 -integer $drecTempList]
		set dscriptList [concat $dscriptList $drecTempList]
	}
	set gscriptList [concat $gscriptList $mgenTempList]
	$logDelayLB insert end "Done"
	$logDelayLB see end
	update idletasks

	#Test Watch for script
	#tk_messageBox -message "DREC Script List:\n  $dscriptList"
	#tk_messageBox -message "MGEN Script List:\n  $gscriptList"
	
	#send scripts to their right place:
	$logDelayLB insert end "Sending mgen script..." ;
	# Mgen only runs in one host so we put it outside the main loop
	$logDelayLB see end
	update idletasks

	#Test Watch for script
	#tk_messageBox -message "Mgen-Script for send command:\n  $gscriptCmd"
	#tk_messageBox -message "Drec-Script for send command:\n  $dscriptCmd"

	# the channel identifier returned by open (fID) is used to write
	# to the command's input pipe (in this case the CAT command)
	if [catch {open "|$gscriptCmd" w} fID] {
		#err message:
		tk_messageBox -message $fID
		$runB config -text "Run Test" -command RunMDelayTest
		return
	}
	#else, transfer data from list to script
	#flush $fID
   
	foreach elem $gscriptList {
		#Write from $elem to a channel($fID)
		puts $fID $elem
	}
	#flush $fID
	if [catch {close $fID} err] {
		tk_messageBox -message $err
		$runB config -text "Run Test" -command RunMDelayTest
		return
	}

	$logDelayLB insert end "Done"

	if $multicast(local,ipv4,$actual) {
		$logDelayLB insert end "Sending drec script to localhost..."
	} else {
		$logDelayLB insert end "Sending drec script to $address..."
	}
	$logDelayLB see end
	update idletasks
   
	if [catch {open "|$dscriptCmd" w} fID] {
		#err message:
		tk_messageBox -message $fID
		$runB config -text "Run Test" -command RunMDelayTest
		return
	}
	#else, transfer data from list to script
	#flush $fID
   
	foreach elem $dscriptList {
		#Write from $elem to a channel($fID)
		puts $fID $elem
	}
	#flush $fID
	if [catch {close $fID} err] {
		tk_messageBox -message $err
		$runB config -text "Run Test" -command RunMDelayTest
		return
	}
	$logDelayLB insert end "Done"
		$logDelayLB see end
	update idletasks

	#update commands:
	append mgenCmd " /tmp/gscript"

	#WAITING DECODE
	append drecCmd " /tmp/dscript /tmp/output.txt"

	##############################################################
	#tk_messageBox -message "Mgen remote-run command:\n  $mgenCmd"
	#tk_messageBox -message "Drec remote-run command:\n  $drecCmd"
	##############################################################

	#change button:
	$runB config -text Stop -command StopMMgen
	
	if $multicast(local,ipv4,$actual) {
		$logDelayLB insert end "Starting drec on localhost..."
	} else {
		$logDelayLB insert end "Starting drec on $address..."
	}
	$logDelayLB see end
	update idletasks

	if [catch {open "|$drecCmd" r+ } dID] \
	{
	#error starting drec
		tk_messageBox -message "$dID"
		$runB config -text "Run Test" -command RunMDelayTest
		update idletasks
		return
	}
	#else:
	$logDelayLB insert end "Done"
	$logDelayLB see end
	update idletasks

	#fconfigure command sets and retrieves options for channels
	#blocking option determines whether I/O operations on the
	#channel can cause the process to block indefinitely. The
	#value of the option must be a proper boolean value. Channels
	#are normally in blocking mode
	fconfigure $dID -blocking false

	#execute a script when a channel becomes readable (when drec starts
	#to receive data from mgen)
	fileevent $dID readable \
	{
		if {[eof $dID]} \
		{
			#to be sure:
			catch {close $dID}
			#catch {StopMMgen}
			#$logDelayLB insert end "Drec-pipe closed"
			$logDelayLB insert end "Mgen stopped"
			$logDelayLB see end
			update idletasks

			#success=1 if Mgen finish well
			if {$success} \
			{
				$logDelayLB insert end "Test completed successfully"
				$logDelayLB see end
				update idletasks
				#XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
				#save the remote files in management station
				SaveMDelayData
			} else {
				$logDelayLB insert end "Test failed"
				$logDelayLB see end
				update idletasks
				$runB config -text "Run Test" -command RunMDelayTest
			}
			#$runB config -text "Run Test" -command RunMDelayTest
			return
		} else {

			#Read a line from a channel -
			# the standar output of Drec execution ??? why ??
			gets $dID line
			$logDelayLB insert end $line
			$logDelayLB see end
			update idletasks
		}
	}
   }
   ### End for: all multicast group hosts
   ### Mgen only runs in one host!!!!!
   $logDelayLB insert end "Starting mgen..."
   $logDelayLB see end
   update idletasks

   # mID = PID of MGEN

   if [catch {open "|$mgenCmd" r+ } mID] {
		#error starting mgen
		tk_messageBox -message $mID
		StopMDrec
		$runB config -text "Run Test" -command RunMDelayTest
		update idletasks
		return
   }
   #else:
   $logDelayLB insert end "Done"
   $logDelayLB see end
   update idletasks

   set success "0"
   #it works well without this:
#   fconfigure $mID -blocking false
   fileevent $mID readable {
		if {[eof $mID]} {
			#mgen is finished:
			if [catch {close $mID} err] {
			#tk_messageBox -message $err
			#$logDelayLB insert end $err
			#$logDelayLB see end
			}
			#tk_messageBox -message "Finalizado.... Succes = $success"
			catch {StopMDrec}
			#$logDelayLB insert end "Mgen-pipe closed"
			$logDelayLB insert end "Mgen stopped"
			$logDelayLB see end
			update idletasks
			return
		} else {
			gets $mID line
			if ![string compare "MGEN: Done." $line] \
			{
				set success 1
			}
			$logDelayLB insert end $line
			$logDelayLB see end
			update idletasks
		}
	}
}

###################################################################
#------------------------------------------------------------------
#                       STOP Multicast DREC
#------------------------------------------------------------------
###################################################################
proc StopMDrec {} \
{
   global runB dID logDelayLB success
   global stopDrecCmd

   #tk_messageBox -message " Waiting to stop DREC....!!!!"
   #$logDelayLB insert end  " - Stopping Drec"
   #$logDelayLB see end
   #update idletasks
   ##stop drec:


   if [catch {open "|$stopDrecCmd" r+ } err] {
      $logDelayLB insert end  $err
      $logDelayLB see end
      update idletasks
      return
   }
   #$logDelayLB insert end  "$success"
   #$logDelayLB see end
   #update idletasks
}


###################################################################
#------------------------------------------------------------------
#                       STOP Multicast MGEN
#------------------------------------------------------------------
###################################################################
proc StopMMgen {} \
{
   global mID logDelayLB stopMgenCmd stopMgenPid
   global runB

   #$logDelayLB insert end  " - Test Stopped from User"
   #$logDelayLB insert end  " - Stopping Mgen"
   #$logDelayLB see end
   update idletasks
   if [catch {open "|$stopMgenCmd" r+ } err] {
      tk_messageBox -message $err
      $logDelayLB insert end  $err
      $logDelayLB see end
      update idletasks
   }

   #tk_messageBox -message "ERRORE 3"
   #StopMDrec
   $runB config -text "Run Test" -command RunMDelayTest
}



###################################################################
#------------------------------------------------------------------
#         SAVE Multicast DELAY DATA (save result .MDLY and other data)
#    save the files from remote station in management station
#------------------------------------------------------------------
###################################################################

proc SaveMDelayData {} \
{
   global success runB logDelayLB getDrecLog
   global sID scenarioDir dlyM fPath

   set fPath ""

   $runB config -text "Wait..." -command {;#do nothing}
   update idletasks

   # box - message
   $logDelayLB insert end "Saving data..."
   $logDelayLB see end
   update idletasks

   #construct path for local file:

   #the "data" directory:
   set path2 data
   #scenario specific directory:
   set path3 [string trim [string trim $scenarioDir(ipv4,multicast)] {/}]
   #file name:
   set path4 [string trim [string trim $dlyM(resultsFile,ipv4)] {/}]

   if [string compare {} $path3] {
   set fPath "$path2/$path3/$path4"
   } else {
   #no scenario dir specified:
   set fPath "$path2/$path4"
   }




   #see if user has specified a file name
   #if not, save to default.dlyM
   if ![string compare {} $path4] \
   {
      append fPath default.dlyM
      #update path
      set dlyM(resultsFile,ipv4) default.dlyM
   }

   #append file extension, if needed
   if [string compare {.dlyM} [file extension $fPath]] \
   {
      append fPath .dlyM
   }

   #find the parent directory for this file, to create it
   set dPath [file dirname $fPath]

   #try to create this dir
   if [catch {file mkdir $dPath} dErr] \
   {
      #error creating directory (if it already exists, no error)
      $logDelayLB insert end "Failed creating directory structure"
      $logDelayLB see end
      $runB config -text "Run Test" -command RunMDelayTest
      update idletasks
      return
   }

   #tk_messageBox -message "dPath : $dPath "
   #tk_messageBox -message "fpath : $fPath "
   #tk_messageBox -message "getDrecLog : $getDrecLog "


   #check if the file already exists:
   if [file exists $fPath] \
   {
      set answer [tk_messageBox -message "File already exists: Overwrite?" \
      -type yesno -icon question]
      if [string compare $answer yes] {
      #user says no:
      $logDelayLB delete 0 end
      $logDelayLB insert end "Change results file name"
      $logDelayLB insert end "and press Save Data button"
      $logDelayLB see end
      $runB config -text "Save Data" -command SaveDelayData
      update idletasks
      return
   }
   #else, it's ok to continue
   }

   #try to open file in write mode:
   if [catch {open $fPath w} iD] \
   {
      #error opening file
      $logDelayLB insert end "Failed opening local file"
      $logDelayLB see end
      $runB config -text "Run Test" -command RunMDelayTest
      update idletasks
      return
   }

   #else, we can write.
   #open pipe to read file:

#RRR
   if [catch {open "|[join $getDrecLog]" r+ } sID] \
   {
      #error opening file
      $logDelayLB insert end "Failed opening remote data file"
      $logDelayLB see end
      $runB config -text "Run Test" -command RunMDelayTest
      update idletasks
      return
   }

   #else:

   #The fcopy command copies data from one I/O channel,
   #inchan to another I/O channel  in background;
   #if an error occur command is executed
   #sID=socket ID
   #fcopy return nbytes and in case error
   #this is the default implementation of the fcopy (see help)
   fcopy $sID $iD -command [list Finished $sID $iD]


   #a test showing that procedure is non-bloking:
   #$logDelayLB insert end "End of SaveDelayData"
   #$logDelayLB see end
}



###################################################################
#------------------------------------------------------------------
#                       FINISHED Multicast
#------------------------------------------------------------------
###################################################################
proc FinishedM {in out bytes {error {}} } \
{
   global logDelayLB runB

   catch {close $in}
   catch {close $out}

   $logDelayLB insert end "$bytes bytes transferred"
   $logDelayLB see end

   if [string length $error] {
      $logDelayLB insert end $error
      $logDelayLB see end
      $runB config -text "Run Test" -command RunMDelayTest
      update idletasks
   } else {
      #at last:
      $logDelayLB insert end "Done"
      $logDelayLB see end
      #!!!!!!!!!!!!!!!!!!!!!!!!!!!
      #in this point, data is saved ok
      ProcessMData
   }
}



###################################################################
#------------------------------------------------------------------
#                       PROCESS Multicast DATA
#------------------------------------------------------------------
###################################################################
proc ProcessMData {} \
{
   global runB logDelayLB fPath mcID
   global mcalcpath4
   global idfilex
   global MCALC_input

   catch {unset outList}

   #announce the beginning
   $logDelayLB insert end "Processing Multicast data..."
   $logDelayLB see end
   update idletasks


   set McalcCmd "$mcalcpath4 $MCALC_input$fPath"
   #tk_messageBox -message "MCALC COMMAND1: \n $McalcCmd"
   #eval exec $McalcCmd
   #eval exec $McalcCmd 
   #set McalcCmd "cat  temp/mcalc.txt"
   #tk_messageBox -message "MCALC COMMAND2: \n $McalcCmd"


   if [catch { open  "|$McalcCmd"} mcID] \
   {
      $logDelayLB insert end "Processing failed"
      $logDelayLB see end
      $runB config -text "Run Test" -command RunMDelayTest
      update idletasks
      return
   }
   set outList ""

   #else, read output in memory:
   fileevent $mcID readable \
   {
      if [eof $mcID] \
      {
         #when done:
         catch {close $mcID}
         SaveMcalcData $outList
         catch {unset outList}
      } else {
         gets $mcID line
         lappend outList $line
      }
   };
};



###################################################################
#------------------------------------------------------------------
#                       SAVE Mulsticast MCALC DATA  (.MMCL)
#------------------------------------------------------------------
###################################################################
proc SaveMMcalcData { outList } \
{

   global runB logDelayLB flowM dlyM mod
   global customParamsD customEndD fPath
   global xID
   global paramU4 pU4 countPU4 


   $runB config -text "Run Test" -command RunMDelayTest

   #update flowM data
   for {set i 1} {$i <= $flowM(number)} {incr i} {
   set flowM(active,$i) no
   set flowM(mod,$i) $mod(number,$i)
   }



   #show mcalc results:
   eval $logDelayLB insert end $outList

   set ok 0
   set lineCount 0

   foreach line $outList {
   if {[scan $line "%s" isFlow] != -1} {
   if ![string compare $isFlow "FLOW:"] {
   #a line announcing a flowM
   incr ok
   #retrieve flowM data
   set count $lineCount

   #flowM number, source and destination:
   scan [lindex $outList $count] "FLOW: %d   SOURCE:%s DESTINATION:%s" \
   fNr src dst

   set mcalc($fNr,source) $src
   set mcalc($fNr,destination) $dst

   #num of packets received
   incr count
   scan [lindex $outList $count] "   Num pkts recvd  : %s" var
   set mcalc($fNr,recvPacketsNumber) $var

   #joined delay:
   incr count
   scan [lindex $outList $count] "   Join delay      : %s" var
   set mcalc($fNr,joinDelay) $var

   #received packet rate:
   incr count
   scan [lindex $outList $count] "   Recv pkt rate   : %s" var
   set mcalc($fNr,recvPacketsRate) $var

   #recv data rate
   incr count
   scan [lindex $outList $count] "   Recv data rate  : %s" var
   set mcalc($fNr,recvDataRate) $var

   #packets dropped:
   incr count
   scan [lindex $outList $count] "   Pkts dropped    : %s" var
   set mcalc($fNr,packetsDropped) $var

   #average delay
   incr count
   scan [lindex $outList $count] "   Ave. Tx Delay   : %s" var
   set mcalc($fNr,averageDelay) $var

   #max delay
   incr count
   scan [lindex $outList $count] "   Max. Tx Delay   : %s" var
   set mcalc($fNr,maxDelay) $var

   #min delay
   incr count
   scan [lindex $outList $count] "   Min. Tx Delay   : %s" var
   set mcalc($fNr,minDelay) $var

   #delay variation:
   incr count
   scan [lindex $outList $count] "   Delay variation : %s" var
   set mcalc($fNr,delayVariation) $var

   #update flowM array:
   set flowM(active,$fNr) yes


   };#end if FLOW:

   incr lineCount
   }

   };#end foreach


   #save data to file

   #set file name: the same as fPath, but different extension:
   #.mcl file

   set mPath [file rootname $fPath]
   append mPath .mcl

   #open file

   if [catch {open $mPath w} xID] {
     $logDelayLB insert end "Failed saving results"
     $logDelayLB see end
     update idletasks
     return
   }
   #else, start writing data:

   for {set i 1} {$i <= $flowM(number)} {incr i} \
   {
      puts $xID " "
      puts $xID "Mgen measure"
      puts $xID "Flow name: $dlyM(title,ipv4) $flowM(name,$i)"
      puts $xID " "
      puts $xID "Flow IPv4 ID $i"
      puts $xID "-Flow Relative Start (msec):  $flowM(start,$i)"
      puts $xID "-Flow Relative Stop  (msec):   $flowM(stop,$i)"
      puts $xID "Pattern  $flowM(pattern,$i)"
      puts $xID "-Send Rate (packets/sec): $flowM(rate,$i)"
      puts $xID "-UDP Packet Size (bytes): $flowM(size,$i)"
      puts $xID "-IP Packet Size (bytes): [expr $flowM(size,$i) +28]"
      puts $xID "Nr. of MODs   $flowM(mod,$i)"
      if $flowM(tosBool,$i) {
         puts $xID "IP TOS Value  $flowM(tos,$i)"
      }
      #rsvp?
      if $flowM(rsvpBool,$i) {
         puts $xID "RSVP Parameters   $flowM(rsvp,$i)"
         puts $xID "RESV Style   $flowM(style,$i)"
         puts $xID "RESV Parameters   $flowM(resv,$i)"
      }

      #absolute start time, if defined:
         if $dlyM(startBool) {
         puts $xID "Mgen Absolute START   $dlyM(start)"
      }

      #####################################################
      # MCALC data
      #####################################################
      puts $xID "Flow Source        $mcalc($i,source)"
      puts $xID "Flow Destination   $mcalc($i,destination)"
      puts $xID "-Received packets: $mcalc($i,recvPacketsNumber)"
      puts $xID "-Join delay (sec): $mcalc($i,joinDelay)"
      puts $xID "-Recv rate (packets/sec): $mcalc($i,recvPacketsRate)"
      puts $xID "-Recv data rate (kbps): $mcalc($i,recvDataRate)"
      puts $xID "-Packets dropped: $mcalc($i,packetsDropped)"
      puts $xID "-Average delay(sec):   $mcalc($i,averageDelay)"
      puts $xID "-Max delay (sec): $mcalc($i,maxDelay)"
      puts $xID "-Min delay (sec): $mcalc($i,minDelay)"
      puts $xID "-Delay variation (sec): $mcalc($i,delayVariation)"

      #custom params:
      set first 1
      set selList [lsort [array names paramU4]]
      for {set count 0} {$count < $countPU4} {incr count} \
      {
         set tmp [lindex $selList $count]
         if [string compare "(parameters)" $tmp] \
         { 
            if { $first == 1 } { puts $xID "Custom parameters:" }
            puts $xID "-$tmp" 
            set first 0 
         }
      }
   }
   catch {close $xID}
   $logDelayLB insert end "Done"
   $logDelayLB see end
   update idletasks

}

