###############################################
# Copyright 2002, 2003 Ren Serral,           #
#                      Roberto Borgione       #
#                                             #
# delayu4b.tcl - Unicast   Delay IPv4 Manager #
#       v2.0 (04-2002)                        #
#       v2.01(10-2002)                        #
###############################################

#   This file is part of NetMeter.
#
#   NetMeter is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   NetMeter is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with NetMeter; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

###################################################################
#------------------------------------------------------------------
#                  RUNDELAYTEST
#------------------------------------------------------------------
###################################################################
#generate the Mgen and Drec Script
#send the Mgen and Drec Script
#execute remote Mgen Drec
#savve the result in local machine

proc RunDelayTest {} \
{
   global sender receiver host
   # dly: general options for all the defined flows
   global dly
   # flow: specific options for the defined flow(param,flow_number)
   global flow
   global mod
   global customParamsD customEndD
   global runB logDelayLB
   global mID dID
   global stopDrecCmd stopMgenCmd
   global success
   global getDrecLog
   global decodepath4
   global SGN
   global _sshCMD

   global debug
   global scripted4
   global errFile

   set randSuffix [randazAZstring 5]
   set errFile "/tmp/netmeterErr$randSuffix"

puts stderr $randSuffix

   if ![info exists scripted4] {
      set scripted4 0
   }

   if $debug {
      puts stderr "RunDelayTest: BEGIN"
   }
   
   #check if there is at least one flow defined:
   if !$flow(number) \
   {
     #0 flows
     if !$scripted4 {
        $logDelayLB insert end "Error must define at least one flow."
        update idletasks
     }
     return
   }

   #set well done test flag to 0:
   set success 0

   #clear status console (message window)
   $runB config -text "Wait..."
   $logDelayLB delete 0 end
   update idletasks

   #find sender and receiver:
   set gscriptFile "/tmp/gscript$randSuffix"
   if $sender(local,ipv4,unicast) {
      set senderHost $host(address,ipv4,unicast)
      set mgenCmd "$dly(mgenPath,ipv4) "
      set gscriptCmd "cat > $gscriptFile"
      set stopMgenCmd "killall $SGN mgen"
   } else {
	set senderHost $sender(address,ipv4,unicast)
	set mgenCmd     "$_sshCMD -l $sender(login,ipv4,unicast) $senderHost $dly(mgenPath,ipv4)"
	set gscriptCmd  "$_sshCMD -l $sender(login,ipv4,unicast) $senderHost \"cat > $gscriptFile\""
	set stopMgenCmd "$_sshCMD -l $sender(login,ipv4,unicast) $senderHost killall $SGN mgen"

	set toSend(login) $sender(login,ipv4,unicast)
	set toSend(host) $senderHost
	set toSend(path) $dly(mgenPath,ipv4)
	if !$scripted4 {
	   $logDelayLB insert end "Verifying remote mgen..."
	   update idletasks
	}
	if { [ verifyRemotePath toSend ] != 0} {
	   if !$scripted4 {
              $logDelayLB insert end "Error with remote mgen..."
	      $runB config -text "Run Test" -command RunDelayTest
	   }
	   return
        }
	if !$scripted4 {
	   $logDelayLB insert end "Remote mgen executable found successfully..."
	   update idletasks
	}
   }

   if $debug {
      if $sender(local,ipv4,unicast) {
         puts stderr "                    sender local"
      }
      puts stderr "                    senderHost=$senderHost"
      puts stderr "                    mgenCmd=$mgenCmd"
      puts stderr "                    gscriptCmd=$gscriptCmd"
      puts stderr "                    stopMgenCmd=$stopMgenCmd\n"
   }

   ##################################################################
   # NOTE over MGEN-DREC SIGINT: 
   # If you don't use SIGINT drec don't finish to write the Received
   # buffer missing the last 5% of output data!!!!
   # also MGEN don't put statistics information :(
   ##################################################################

   set dscriptFile "/tmp/dscript$randSuffix"
   set outputFile "/tmp/output.txt$randSuffix"
   set outputFileBin "/tmp/output.bin$randSuffix"
   if $receiver(local,ipv4) {
      set receiverHost $host(address,ipv4,unicast)
      set drecCmd $dly(drecPath,ipv4)
      set dscriptCmd "cat > $dscriptFile"
      set stopDrecCmd "killall $SGN drec"
      #OLD MGEN4 version= set getDrecLog  "\"cat /tmp/logDrec\""
      set getDrecLog  "cat $outputFile"
      #Waiting the DECODE set getDrecLog  "$decodepath4 $outputFileBin | cat"
   } else {
      set receiverHost $receiver(address,ipv4,unicast)
      set drecCmd     "$_sshCMD -l $receiver(login,ipv4)  $receiverHost $dly(drecPath,ipv4) "
      set dscriptCmd  "$_sshCMD -l $receiver(login,ipv4)  $receiverHost \"cat > $dscriptFile\""
      #OLD MGEN4 version= set getDrecLog  "$_sshCMD -l $receiver(login,ipv4)  $receiverHost cat /tmp/logDrec"
      set getDrecLog  "$_sshCMD -l $receiver(login,ipv4)  $receiverHost cat $outputFile"
      #Waiting the DECODE set getDrecLog  "$_sshCMD -l $receiver(login,ipv4)  $receiverHost $decodepath4 $outputFileBin | cat"
      set stopDrecCmd "$_sshCMD -l $receiver(login,ipv4)  $receiverHost killall $SGN drec"
	set toSend(login) $sender(login,ipv4,unicast)

	set toSend(host) $receiverHost
	set toSend(path) $dly(drecPath,ipv4)
	if !$scripted4 {
	   $logDelayLB insert end "Verifying remote drec..."
	   update idletasks
	}
	if { [ verifyRemotePath toSend ] != 0} {
		if !$scripted4 {
		   $logDelayLB insert end "Error with remote drec..."
		   $runB config -text "Run Test" -command RunDelayTest
		}
		return
	}
	if !$scripted4 {
	   $logDelayLB insert end "Remote drec executable found successfully..."
	   update idletasks
	}
   }

   if $debug {
      if $receiver(local,ipv4) {
         puts stderr "                    receiver local"
      }
      puts stderr "                    receiverHost=$receiverHost"
      puts stderr "                    drecCmd=$drecCmd"
      puts stderr "                    dscriptCmd=$dscriptCmd"
      puts stderr "                    getDrecLog=$getDrecLog"
      puts stderr "                    stopDrecCmd=$stopDrecCmd\n"
   }

   set receiverHost $host(address,ipv4,unicast)
   
   if !$scripted4 {
      $logDelayLB insert end "Verifying local decode..."
      update idletasks
   }
   if { [ verifyLocalPath $decodepath4 0 ] != 0 } {
      if $debug {
         puts stderr "                    decode NOT found, not necessarily serious, supposing non coded MGEN version"
      }
      set decodepath4 "/bin/cat "
   }

   ##construct lists with script commands
   #put a message on window
   if !$scripted4 {
      $logDelayLB insert end "Building scripts..."
      $logDelayLB see end
      update idletasks
   }

   lappend gscriptList "# mgen launching script"
   lappend dscriptList "# drec launching script"

   #if we have a start command
   if $dly(startBool) {
       #[trim]->kill space & tab chars
       #if ($dly(start)!="") then .....
       if {[string trim $dly(start)] != ""} {
          lappend gscriptList "START $dly(start)"
          lappend dscriptList "START $dly(start)"
       }
   }

   #if we have a srcPort command for mgen
   #the port for the source flow is ONE
   #for drec we have to set the ports for all the flows
   if $dly(srcPortBool) {
       if {[string trim $dly(srcPort)] != ""} {
         lappend gscriptList "PORT $dly(srcPort)"
       }
   }

   #interfaces:
   #mgen interface
   if $sender(local,ipv4,unicast) {
      set sInterface $host(interface,ipv4,unicast)
   } else {
      set sInterface $sender(interface,ipv4,unicast)
   }

   if {[string trim $sInterface] != ""} {
     append mgenCmd " -i $sInterface "
   }

   #drec interface
   if $receiver(local,ipv4) {
      set rInterface $host(interface,ipv4,unicast)
   } else {
      set rInterface $receiver(interface,ipv4)
   }

   if {[string trim $rInterface] != ""} {
      lappend dscriptList "INTERFACE $rInterface"
   }

   #more flow with different destination port
   #ports listened by drec:
   for {set i 1} {$i <= $flow(number)} {incr i} {
      lappend portList $flow(port,$i)
   }

   #this loop will run at least once,
   #so portList will certainly exist
   lappend dscriptList "PORT [join $portList {,}]"

   #Test Watch for script starts time and ports
   #tk_messageBox -message "DREC Script List:\n  $dscriptList"
   #tk_messageBox -message "MGEN Script List:\n  $gscriptList"

#--------------------------------------------------------------

   #deal with flows:
   #construct two temporary lists with commands that
   #will be sorted chronologically afterwards (see the start time)
   lappend drecTempList ""
   for {set i 1} {$i <= $flow(number)} {incr i} \
   {
      set unresv 0
      #temporary line
      set tempLine "$flow(start,$i) $i ON $receiverHost:$flow(port,$i) \
      $flow(pattern,$i) $flow(rate,$i) $flow(size,$i)"
      #the flow(i) use RSVP parameters ?
      if $flow(rsvpBool,$i) {
         if {[string trim $flow(rsvp,$i)] != ""} \
         {
            append tempLine " RSVP $flow(rsvp,$i)"
            #RESV:
            lappend drecTempList "$flow(start,$i) RESV $receiverHost:$flow(port,$i) \
               $flow(style,$i) $flow(resv,$i)"
            lappend drecTempList "$flow(stop,$i) UNRESV $receiverHost:$flow(port,$i)"
            set unresv 1
         }
      }
      #the flow(i) use TYPE of SERVICE parameters ?
      if $flow(tosBool,$i) {
         if {[string trim $flow(tos,$i)] != ""} \
         {
            append tempLine " TOS $flow(tos,$i)"
         }
      }
      lappend mgenTempList $tempLine

      #note: a generic flux(i) is associeted a $mod(number,$i) modify flows
      #deal with MOD data
      for {set j 1} {$j <= $mod(number,$i)} {incr j} \
      {
         set tempLine "$mod(start,$j,$i) $i MOD $receiverHost:$flow(port,$i) \
             $mod(pattern,$j,$i) $mod(rate,$j,$i) $mod(size,$j,$i)"
         #the mod(i) use RSVP parameters ?
         if $mod(rsvpBool,$j,$i) {
            if {[string trim $mod(rsvp,$j,$i)] != ""}\
            {
               append tempLine " RSVP $mod(rsvp,$j,$i)"
               lappend drecTempList "$mod(start,$j,$i) RESV $receiverHost:$flow(port,$i) \
                  $mod(style,$j,$i) $mod(resv,$j,$i)"
               if {!$unresv} {
                  lappend drecTempList "$flow(stop,$i) UNRESV $receiverHost:$flow(port,$i)"
                  set unresv 1
               }
            }
         }
         #the mod(i) use TYPE of SERVICE parameters ?
         if $mod(tosBool,$j,$i) {
           if {[string trim $mod(tos,$j,$i)] != ""} {
             append tempLine " TOS $mod(tos,$j,$i)"
           }
         }
         lappend mgenTempList $tempLine
      }
      lappend mgenTempList "$flow(stop,$i) $i OFF"
   }
   #sort mgenTempList and drecTempList
   if [catch {lsort -index 0 -integer $mgenTempList} mgenTempList] \
   {
		$logDelayLB insert end "ERROR - All start-stop flow values must be integers"
		$runB config -text "Run Test" -command RunDelayTest
		update idletasks
		tk_messageBox -message "All flows start-stop values\nmust be integers"
		return
   }

   #all time values are integers
   #no need to catch next command (uses the same start-stop values)
   if {[string trim $drecTempList] != ""} \
   {
      set drecTempList [lreplace $drecTempList 0 0]
      set drecTempList [lsort -index 0 -integer $drecTempList]
      set dscriptList [concat $dscriptList $drecTempList]
   }
   set gscriptList [concat $gscriptList $mgenTempList]
   if !$scripted4 {
      $logDelayLB insert end "Done"
      $logDelayLB see end
      update idletasks
#--------------------------------------------------------------
      $logDelayLB insert end "Sending mgen script..."
      $logDelayLB see end
      update idletasks
   }
   #send scripts to their right place:

   # the channel identifier returned by open (fID) is used to write
   # to the command's input pipe (in this case the CAT command)
   if $debug {
      puts stderr "               Sending mgen script Command: $gscriptCmd"
   }

   if [catch {open "|$gscriptCmd 2> $errFile" w} fID] \
   {
      #err message:
      tk_messageBox -message $fID
      $runB config -text "Run Test" -command RunDelayTest
      return
   }
   #else, transfer data from list to script
   #flush $fID

   foreach elem $gscriptList \
   {
      #Write from $elem to a channel($fID)
      puts $fID $elem
   }
   #flush $fID
   if [catch {close $fID} err] \
   {
      tk_messageBox -message $err
      $runB config -text "Run Test" -command RunDelayTest
      return
   }

   if !$scripted4 {
      $logDelayLB insert end "Done"
      $logDelayLB insert end "Sending drec script..."
      $logDelayLB see end
      update idletasks
   }
   if $debug {
       puts stderr "               Finished Sending mgen script" 
       puts stderr "               Sending drec script Command: $dscriptCmd" 
   }

   if [catch {open "|$dscriptCmd 2> $errFile" w} fID] \
   {
      #err message:
      tk_messageBox -message $fID
      $runB config -text "Run Test" -command RunDelayTest
	  $logDelayLB insert end "ERROR - Sending drec script."
      update idletasks
      return
   }
   #else, transfer data from list to script
   #flush $fID

   foreach elem $dscriptList \
   {
      #Write from $elem to a channel($fID)
      puts $fID $elem
   }
   #flush $fID
   if [catch {close $fID} err] \
   {
	if !$scripted4 {
	   tk_messageBox -message $err
	   $logDelayLB insert end "ERROR - Sending drec script."
	}
	$runB config -text "Run Test" -command RunDelayTest
	return
   }
   if !$scripted4 {
      $logDelayLB insert end "Done"
      $logDelayLB see end
      update idletasks
   }

   #update commands:
   append mgenCmd " $gscriptFile"
   append drecCmd " $dscriptFile $outputFile"

   ##############################################################
   #tk_messageBox -message "Mgen remote-run command:\n  $mgenCmd"
   #tk_messageBox -message "Drec remote-run command:\n  $drecCmd"
   ##############################################################

   #change button:
   $runB config -text Stop -command StopMgen
   if !$scripted4 {
      $logDelayLB insert end "Starting drec..."
      $logDelayLB see end
      update idletasks
   }
   if $debug {
       puts stderr "               Starting drec with command: $drecCmd" 
   }

   if [catch {open "|$drecCmd 2> $errFile" r+ } dID] \
   {
   #error starting drec
      if $debug {
         puts stderr "               +++++ERROR+++++ Starting drec" 
         puts stderr "$dID"
      }
      if !$scripted4 {
	  tk_messageBox -message "$dID"
	  $logDelayLB insert end "ERROR - Starting drec: $dID"
	  update idletasks
      }
      $runB config -text "Run Test" -command RunDelayTest
      return
   }
   #else:
   if !$scripted4 {
      $logDelayLB insert end "Delaying"
      $logDelayLB see end
      update idletasks
      after 10000
      $logDelayLB insert end "Done"
      $logDelayLB see end
      update idletasks
   } else {
      after 10000
   }

   #fconfigure command sets and retrieves options for channels
   #blocking option determines whether I/O operations on the
   #channel can cause the process to block indefinitely. The
   #value of the option must be a proper boolean value. Channels
   #are normally in blocking mode
   fconfigure $dID -blocking false

   #execute a script when a channel becomes readable (when drec starts
   #to receive data from mgen)
   fileevent $dID readable {
      if {[eof $dID]} {
        #to be sure:
        catch {StopMgen}
	if !$scripted4 {
		$logDelayLB insert end "Mgen stopped due a drec error"
		$logDelayLB see end
		update idletasks
	}
        catch {close $dID}
	return
      } else {
        gets $dID line
	if {$success!=1} {
		 if !$scripted4 {
        	    $logDelayLB insert end $line
	            $logDelayLB see end
        	    update idletasks
		 }
	}
      }
   }
   if $debug {
      puts stderr "               Starting mgen with command: $mgenCmd"
   }

   if !$scripted4 {
      $logDelayLB insert end "Starting mgen..."
      $logDelayLB see end
      update idletasks
   }

   if [catch {open "|$mgenCmd 2> $errFile" r+ } mID] {
      if $debug {
         puts stderr "               +++++ERROR+++++ Starting MGEN, program Output:"
         puts stderr "$mID"
      }
      #error starting mgen
      StopDrec
      if !$scripted4 {
        tk_messageBox -message $mID
        $runB config -text "Run Test" -command RunDelayTest
        update idletasks
      }
      return
   }
   #else:
   if !$scripted4 {
      $logDelayLB insert end "Done"
      $logDelayLB see end
      update idletasks
   }

   set success "0"
   set finish_mgen 0
   fconfigure $mID -blocking true
   while {![eof $mID]} {
	gets $mID line
	if ![string compare "MGEN: Done." $line] {
		if $debug {
			puts stderr "             Test completed successfully"
		}
		if !$scripted4 {
			$logDelayLB insert end "Test completed successfully"
			$logDelayLB see end
			update idletasks
		}
		catch {close $dID}
		StopDrec
		SaveDelayData
		set finish_mgen 1
	}
	if !$scripted4 {
		$logDelayLB insert end $line
		$logDelayLB see end
		update idletasks
	}
   }
   if !$finish_mgen {
	catch {close $dID}
	StopDrec
	$logDelayLB insert end stderr "Verify the destination Mgen IP in the Main Options Tab"
        $runB config -text "Run Test" -command RunDelayTest
	update idletasks
   }
   if [catch {close $mID} err] {
   	if $debug {
		puts stderr "               ERROR: $err"
		puts stderr "                      Verify the destination Mgen IP in the Main Options Tab"
	}
	if !$scripted4 {
		$logDelayLB insert end $err
		$logDelayLB see end
		tk_messageBox -message $err
	}
   }
   if $debug {
	puts stderr "               mgen Stopped"
   }
   if !$scripted4 {
	$logDelayLB insert end "MGen stopped"
	$logDelayLB see end
        $runB config -text "Run Test" -command RunDelayTest
	update idletasks
   }	
   if $debug {
      puts stderr "RunDelayTest: END"
   }
}

###################################################################
#------------------------------------------------------------------
#                       STOPDREC
#------------------------------------------------------------------
###################################################################
proc StopDrec {} \
{
   global runB dID logDelayLB success
   global stopDrecCmd
   global errFile

   global debug scripted4

   if $debug {
       puts stderr "StopDrec BEGIN:"
       puts stderr "              Stopping Drec"
       puts stderr "              Sleeping 10 seconds for packets to arrive"
   }
   if !$scripted4 {
      $logDelayLB insert end "Delaying drec stop"
      $logDelayLB see end
      update idletasks
   }
   after 10000

   if [catch {open "|$stopDrecCmd 2> $errFile" r+ } err] {
      if $debug {
         puts stderr "              ERROR: $err"
      }
      if !$scripted4 {
         $logDelayLB insert end  $err
         $logDelayLB see end
         update idletasks
      }
      return
   }
   if $debug {
       puts stderr "              Drec succesfully stoped"
       puts stderr "StopDrec END:"
   }
}

###################################################################
#------------------------------------------------------------------
#                       STOPMGEN
#------------------------------------------------------------------
###################################################################
proc StopMgen {} \
{
   global mID logDelayLB stopMgenCmd stopMgenPid
   global runB
   global errFile

   global debug

   if $debug {
       puts stderr "StopMgen BEGIN:"
       puts stderr "              Stopping Mgen"
   }
   if [catch {open "|$stopMgenCmd 2> $errFile" r+ } err] {
      if !$scripted4 {
         tk_messageBox -message $err
         $logDelayLB insert end  $err
         $logDelayLB see end
         update idletasks
      }
   }

   if $debug {
       puts stderr "              Mgen succesfully stopped"
       puts stderr "StopMgen END:"
   }

   if !$scripted4 {
      $runB config -text "Run Test" -command RunDelayTest
   }
}

###################################################################
#------------------------------------------------------------------
#         SAVEDELAYDATA (save result .DLY and other data)
#    save the files from remote station in management station
#------------------------------------------------------------------
###################################################################
proc SaveDelayData {} \
{
   global tempDecode
   global success runB logDelayLB getDrecLog
   global sID scenarioDir dly fPath

   global debug scripted4

   set fPath ""

   if $debug {
      puts stderr "SaveDelayData: BEGIN"
   }

   if !$scripted4 {
      $runB config -text "Wait..." -command {;#do nothing}
      update idletasks

      # box - message
      $logDelayLB insert end "Saving data..."
      $logDelayLB see end
      update idletasks
   }
   #construct path for local file:
   #the "data" directory:
   set path2 data
   #scenario specific directory:
   set path3 [string trim [string trim $scenarioDir(ipv4,unicast)] {/}]
   #file name:
   set path4 [string trim [string trim $dly(resultsFile,ipv4)] {/}]

   if [string compare {} $path3] {
        set fPath "$path2/$path3/$path4"
   } else {
	#no scenario dir specified:
	set fPath "$path2/$path4"
   }

   #see if user has specified a file name
   #if not, save to default.dly
   if ![string compare {} $path4] \
   {
      append fPath default.dly
      #update path
      set dly(resultsFile,ipv4) default.dly
   }

   #append file extension, if needed
   if [string compare {.dly} [file extension $fPath]] \
   {
      append fPath .dly
   }

   #find the parent directory for this file, to create it
   set dPath [file dirname $fPath]

   #try to create this dir
   if [catch {file mkdir $dPath} dErr] \
   {
      if $debug {
         puts stderr "                     ERROR: Failed creating directory structure"
      }
      #error creating directory (if it already exists, no error)
      if !$scripted4 {
         $logDelayLB insert end "Failed creating directory structure"
         $logDelayLB see end
         $runB config -text "Run Test" -command RunDelayTest
         update idletasks
      }
      return
   }

   if $debug {
      puts stderr "                    Saving to: $fPath"
      puts stderr "                    With: $getDrecLog"
   }

   #check if the file already exists:
   if [file exists $fPath] \
   {
      if !$scripted4 {
         set answer [tk_messageBox -message "File already exists: Overwrite?" \
           -type yesno -icon question]
      } else {
	 set answer yes
      }
      if [string compare $answer yes] {
         if !$scripted4 {
		#user says no:
		$logDelayLB delete 0 end
		$logDelayLB insert end "Change results file name"
		$logDelayLB insert end "and press Save Data button"
		$logDelayLB see end
		$runB config -text "Save Data" -command SaveDelayData
		update idletasks
	 }
	 return
     }
   #else, it's ok to continue
   }
   
   set tempDecode "temp/p_decode.tmp"
   if $debug {
       puts stderr "                    Decoding drec output with Command: $tempDecode"
   }
   #try to open file in write mode:
   if [catch {open $tempDecode w} iD] {
      #error opening file
      $logDelayLB insert end "Failed opening local file $tempDecode. Check directori permissions"
      $logDelayLB see end
      $runB config -text "Run Test" -command RunDelayTest
      update idletasks
      return
   }

   #else, we can write.
   #open pipe to read file:
   if [catch {open "|[join $getDrecLog]" r+ } sID] {
      #error opening file
      $logDelayLB insert end "Failed opening remote data file"
      $logDelayLB see end
      $runB config -text "Run Test" -command RunDelayTest
      update idletasks
      return
   }
   #else:

   #The fcopy command copies data from one I/O channel,
   #inchan to another I/O channel  in background;
   #if an error occur command is executed
   #sID=socket ID
   #fcopy return nbytes and in case error
   #this is the default implementation of the fcopy (see help)
   fconfigure $sID -translation binary
   fconfigure $iD -translation binary

#   fcopy $sID $iD -command [list Finished $sID $iD]
   set bytes [fcopy $sID $iD]
   Finished $sID $iD $bytes

   if $debug {
      puts stderr "SaveDelayData: END"
   }
}

###################################################################
#------------------------------------------------------------------
#                       FINISHED
#------------------------------------------------------------------
###################################################################
proc Finished {in out bytes {error {}} } \
{
   global logDelayLB runB scripted4 debug

   catch {close $in}
   catch {close $out}

   if !$scripted4 {
      $logDelayLB insert end "$bytes bytes transferred"
      $logDelayLB see end
   }

	if {$bytes == 0} {
		if $debug {
			puts stderr "                 -------ERROR--------: Test Failed empty data file found!"
		}
		if !$scripted4 {
			tk_messageBox -message "Test Failed empty data file found\nPlease check Source, Destination or Directory Permissions"
			$runB config -text "Run Test" -command RunDelayTest6
			update idletasks
		}
		return
	}

   if [string length $error] {
      if !$scripted4 {
         $logDelayLB insert end $error
         $logDelayLB see end
         $runB config -text "Run Test" -command RunDelayTest
         update idletasks
      }
   } else {
      if !$scripted4 {
         $logDelayLB insert end "Done"
         $logDelayLB see end
      }
      ProcessData
   }
}

###################################################################
#------------------------------------------------------------------
#                       PROCESSDATA
#------------------------------------------------------------------
###################################################################
proc ProcessData {} \
{
   global runB logDelayLB fPath mcID
   global mcalcpath4 decodepath4
   global idfilex
   global MCALC_input
   global tempDecode scripted4

   global debug
   
   catch {unset outList}

   if $debug {
		puts stderr "ProcessData: BEGIN"
   }

   #announce the beginning
   if !$scripted4 {
      $logDelayLB insert end "Processing data..."
      $logDelayLB see end
      update idletasks
   }

   if [info exists tempDecode] {
      set tempCmd "$decodepath4 $tempDecode > $fPath "
   } else {
      set tempCmd "/bin/true";
   }
   if {[ verifyMcalc $mcalcpath4 ] != 0 } {
      if $debug {
         puts stderr "Good, your mcalc version rocks!"
      }
      set McalcCmd "$mcalcpath4 $MCALC_input$fPath "
   } else {
      if $debug {
         puts stderr "Non directly supported mcalc, problems can occur, continuing anyway"
      }
      set McalcCmd "$mcalcpath4 $fPath "
   }
   if $debug {
		puts stderr "                  Decode: $tempCmd"
		puts stderr "                  Mcalc:  $McalcCmd"
   }
   
   if [catch {eval exec $tempCmd} mcID] \
   {
	if $debug {
		puts stderr "                  ERROR - Decoding results: $mcID"
	}
      if !$scripted4 { 
         $logDelayLB insert end "Processing Decode failed"
         $logDelayLB see end
         $runB config -text "Run Test" -command RunDelayTest
         update idletasks
      }
      return
   }
   if [catch { open  "|$McalcCmd"} mcID] {
	  if $debug {
		puts stderr "                  ERROR - MCalc-ing results: $mcID"
	  }
      if !$scripted4 {
         $logDelayLB insert end "Processing MCalc failed"
         $logDelayLB see end
         $runB config -text "Run Test" -command RunDelayTest
         update idletasks
      }
      return
   }
   
   set outList ""
   fconfigure $mcID -blocking true
   #else, read output in memory:
   while {![eof $mcID]} {
         gets $mcID line
         lappend outList $line
   }
   #when done:
   catch {close $mcID}
   SaveMcalcData $outList
   catch {unset outList}
   if $debug {
		puts stderr "ProcessData: END"
   }
}

###################################################################
#------------------------------------------------------------------
#                       SAVE MCALC DATA  (.MCL)
#------------------------------------------------------------------
###################################################################
proc SaveMcalcData { outList } \
{
   global runB logDelayLB flow dly mod
   global customParamsD customEndD fPath
   global xID
   global paramU4 pU4 countPU4 

   global debug

   $runB config -text "Run Test" -command RunDelayTest

   if $debug {
	 	puts stderr "SaveMcalcData: BEGIN"
	}
   
   #update flow data
   for {set i 1} {$i <= $flow(number)} {incr i} {
		set flow(active,$i) no
		set flow(mod,$i) $mod(number,$i)
   }

   #show mcalc results:
   eval $logDelayLB insert end $outList

   set ok 0
   set lineCount 0

   foreach line $outList {
	if {[scan $line "%s" isFlow] != -1} {
		if ![string compare $isFlow "FLOW:"] {
			#a line announcing a flow
			incr ok
			#retrieve flow data
			set count $lineCount

			#flow number, source and destination:
			scan [lindex $outList $count] "FLOW: %d   SOURCE:%s DESTINATION:%s" \
			fNr src dst

			set mcalc($fNr,source) $src
			set mcalc($fNr,destination) $dst

			#num of packets received
			incr count
			scan [lindex $outList $count] "   Num pkts recvd  : %s" var
			set mcalc($fNr,recvPacketsNumber) $var

			#joined delay:
			incr count
			scan [lindex $outList $count] "   Join delay      : %s" var
			set mcalc($fNr,joinDelay) $var

			#received packet rate:
			incr count
			scan [lindex $outList $count] "   Recv pkt rate   : %s" var
			set mcalc($fNr,recvPacketsRate) $var

			#recv data rate
			incr count
			scan [lindex $outList $count] "   Recv data rate  : %s" var
			set mcalc($fNr,recvDataRate) $var

			#packets dropped:
			incr count
			scan [lindex $outList $count] "   Pkts dropped    : %s" var
			set mcalc($fNr,packetsDropped) $var

			#average delay
			incr count
			scan [lindex $outList $count] "   Ave. Tx Delay   : %s" var
			set mcalc($fNr,averageDelay) $var

			#max delay
			incr count
			scan [lindex $outList $count] "   Max. Tx Delay   : %s" var
			set mcalc($fNr,maxDelay) $var

			#min delay
			incr count
			scan [lindex $outList $count] "   Min. Tx Delay   : %s" var
			set mcalc($fNr,minDelay) $var

			#delay variation:
			incr count
			scan [lindex $outList $count] "   Delay variation : %s" var
			set mcalc($fNr,delayVariation) $var

			#update flow array:
			set flow(active,$fNr) yes
		};#end if FLOW:
		incr lineCount
	}
   };#end foreach
   
   #save data to file

   #set file name: the same as fPath, but different extension:
   #.mcl file

   set mPath [file rootname $fPath]
   append mPath .mcl

   #open file

   if [catch {open $mPath w} xID] {
     if $debug {
	 	puts stderr "                     ERROR - Saving results: $xID"
	 }
     $logDelayLB insert end "Failed saving results"
     $logDelayLB see end
     update idletasks
     return
   }
   #else, start writing data:

   for {set i 1} {$i <= $flow(number)} {incr i} \
   {
      puts $xID " "
      puts $xID "Mgen measure"
      puts $xID "Flow name: $dly(title,ipv4) $flow(name,$i)"
      puts $xID " "
      puts $xID "Flow IPv4 ID $i"
      puts $xID "-Flow Relative Start (msec):  $flow(start,$i)"
      puts $xID "-Flow Relative Stop  (msec):   $flow(stop,$i)"
      puts $xID "Pattern  $flow(pattern,$i)"
      puts $xID "-Send Rate (packets/sec): $flow(rate,$i)"
      puts $xID "-UDP Packet Size (bytes): $flow(size,$i)"
      puts $xID "-IP Packet Size (bytes): [expr $flow(size,$i) +28]"
      puts $xID "Nr. of MODs   $flow(mod,$i)"
      if $flow(tosBool,$i) {
         puts $xID "IP TOS Value  $flow(tos,$i)"
      }
      #rsvp?
      if $flow(rsvpBool,$i) {
         puts $xID "RSVP Parameters   $flow(rsvp,$i)"
         puts $xID "RESV Style   $flow(style,$i)"
         puts $xID "RESV Parameters   $flow(resv,$i)"
      }

      #absolute start time, if defined:
         if $dly(startBool) {
         puts $xID "Mgen Absolute START   $dly(start)"
      }

      #####################################################
      # MCALC data
      #####################################################
      puts $xID "Flow Source        $mcalc($i,source)"
      puts $xID "Flow Destination   $mcalc($i,destination)"
      puts $xID "-Received packets: $mcalc($i,recvPacketsNumber)"
      puts $xID "-Join delay (sec): $mcalc($i,joinDelay)"
      puts $xID "-Recv rate (packets/sec): $mcalc($i,recvPacketsRate)"
      puts $xID "-Recv data rate (kbps): $mcalc($i,recvDataRate)"
      puts $xID "-Packets dropped: $mcalc($i,packetsDropped)"
      puts $xID "-Average delay(sec):   $mcalc($i,averageDelay)"
      puts $xID "-Max delay (sec): $mcalc($i,maxDelay)"
      puts $xID "-Min delay (sec): $mcalc($i,minDelay)"
      puts $xID "-Delay variation (sec): $mcalc($i,delayVariation)"

      #custom params:
      set first 1
      set selList [lsort [array names paramU4]]
      for {set count 0} {$count < $countPU4} {incr count} \
      {
         set tmp [lindex $selList $count]
         if [string compare "(parameters)" $tmp] \
         { 
            if { $first == 1 } { puts $xID "Custom parameters:" }
            puts $xID "-$tmp" 
            set first 0 
         }
      }
   }
   catch {close $xID}
   $logDelayLB insert end "Done"
   $logDelayLB see end
   update idletasks
   
    if $debug {
	 	puts stderr "SaveMcalcData: END"
	}
}

