###############################################
# Copyright 2002, 2003 Ren Serral,           #
#                      Roberto Borgione       #
#                                             #
# main.tcl - Main Manager                     #
#       v2.0 (04-2002)                        #
#       v2.01(10-2002)                        #
###############################################

#   This file is part of NetMeter.
#
#   NetMeter is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   NetMeter is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with NetMeter; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

###################################################################
#------------------------------------------------------------------
#                  M A I N - S E T U P  (page1)
#------------------------------------------------------------------
###################################################################
proc MainSetup {} \
{
   global ConfInfo
   global GlobalFileConf
   global fL fR
   global page		;#container pages handlers
   global ipVersion	;#4 or 6
   global sender	;#sender parameters (address, interface,login,password,local) for each scenario type
   global receiver	;#receiver parameters (mainly for unicast case)
   global addressType	;#unicast or multicast receiver address
   global multicast	;#array for unicast IPs in multicast group
   global multicastEnd	;#number of current unicast IPs in group
   global newState	;#current scenario type: (unicast-multicast, ipv4-ipv6)
   global host		;#management station params (address and interface)
   global scenarioDir	;#directory to save results
   global x		;#listbox handle in page1 (multicast case)

# Ren
   global debug		;#Debug output to stderr

   set debug 1

   ############################################################
   ### LOADING THE IMAGES for WINDOWS
   ############################################################
#   image create photo uimage -file [file join sources/uni.gif]
#   image create photo mimage -file [file join sources/multi.gif]

   ############################################################
   ### TK part for PAGE 1 - MAIN SETUP
   ############################################################
   pack $fL $fR -side left -fill both -expand true
   set   sF  [frame $fL.scenarioF -bd 2 -relief groove]
   pack $sF -fill x
   frame $sF.ipVersionFrame
   label $sF.ipVersionFrame.protocolLabel -text "Protocol\nVersion"
   radiobutton $sF.ipVersionFrame.r4 -text "IPv4" -variable ipVersion \
         -value "ipv4" -command Select -bd 2 -relief groove
   radiobutton $sF.ipVersionFrame.r6 -text "IPv6" -variable ipVersion \
         -value "ipv6" -command Select -bd 2 -relief groove

   pack $sF.ipVersionFrame -pady 2 -side left
   pack $sF.ipVersionFrame.protocolLabel -fill x -expand true -padx 3 \
        -pady 2 -side left
   pack $sF.ipVersionFrame.r4 $sF.ipVersionFrame.r6 -padx 2

   frame $sF.aTypeFrame
   label $sF.aTypeFrame.aTypeLabel -text "Address\nType"
   radiobutton $sF.aTypeFrame.rU -text "Unicast" -anchor w\
       -variable addressType -value "unicast" -command Select -bd 2 -relief groove
   radiobutton $sF.aTypeFrame.rM -text "Multicast" -anchor w\
       -variable addressType -value "multicast" -command Select -bd 2 -relief groove -state disabled

   pack $sF.aTypeFrame -fill x -pady 2 -side left
   pack $sF.aTypeFrame.aTypeLabel -fill x -expand true -padx 3 \
       -pady 2 -side left
   pack $sF.aTypeFrame.rU $sF.aTypeFrame.rM \
       -padx 2 -expand true -fill x


   ###################################
   ###  frame for sender parameters
   ###################################
   frame $fL.senderFrame -bd 2 -relief groove
   pack $fL.senderFrame -fill x

   #address:
   frame $fL.senderFrame.saFrame
   label $fL.senderFrame.saFrame.saL -text "Sender Address:"
   entry $fL.senderFrame.saFrame.saE -textvariable sender(address,$ipVersion,$addressType)
   pack  $fL.senderFrame.saFrame -fill x -expand true
   pack  $fL.senderFrame.saFrame.saL -fill x -padx 2 -pady 2 -side left
   pack  $fL.senderFrame.saFrame.saE -fill x -padx 2 -pady 2 -side right

   #interface:
   frame $fL.senderFrame.siFrame
   label $fL.senderFrame.siFrame.siL -text "Sender Interface:"
   entry $fL.senderFrame.siFrame.siE \
        -textvariable sender(interface,$ipVersion,$addressType) -width 13
   pack $fL.senderFrame.siFrame -fill x -expand true
   pack $fL.senderFrame.siFrame.siL -padx 2 -pady 2 -side left
   pack $fL.senderFrame.siFrame.siE -padx 2 -pady 2 -side right

   #user (login name) for remote commands execution
   frame $fL.senderFrame.slFrame
   label $fL.senderFrame.slFrame.slL -text "Sender Login:"
   entry $fL.senderFrame.slFrame.slE \
         -textvariable sender(login,$ipVersion,$addressType) -width 13
   pack $fL.senderFrame.slFrame -fill x -expand true
   pack $fL.senderFrame.slFrame.slL -side left -padx 2 -pady 2
   pack $fL.senderFrame.slFrame.slE -padx 2 -pady 2 -side right

   #password:
   frame $fL.senderFrame.spFrame
   label $fL.senderFrame.spFrame.spL -text "Sender Password:"
   entry $fL.senderFrame.spFrame.spE \
         -textvariable sender(password,$ipVersion,$addressType) -show "*" -width 13
   pack $fL.senderFrame.spFrame -fill x -expand true
   pack $fL.senderFrame.spFrame.spL -padx 2 -pady 2 -side left
   pack $fL.senderFrame.spFrame.spE -padx 2 -pady 2 -side right

   #checkbutton: local sender?
   checkbutton $fL.senderFrame.localSender -text "local" \
         -variable sender(local,$ipVersion,$addressType) -command LocalSender
   pack $fL.senderFrame.localSender -pady 2

   ###################################
   ### frame for receiver parameters
   ###################################
   frame $fL.receiverFrame -bd 2 -relief groove
   pack $fL.receiverFrame -pady 2 -fill x

   #address:
   frame $fL.receiverFrame.raFrame
   label $fL.receiverFrame.raFrame.raL -text "Receiver Address:"
   entry $fL.receiverFrame.raFrame.raE \
         -textvariable receiver(address,$ipVersion,$addressType)
   pack $fL.receiverFrame.raFrame -fill x -expand true
   pack $fL.receiverFrame.raFrame.raL -fill x -padx 2 -pady 2 -side left
   pack $fL.receiverFrame.raFrame.raE -fill x -padx 2 -pady 2 -side right

   #interface:
   frame $fL.receiverFrame.riFrame
   label $fL.receiverFrame.riFrame.riL -text "Receiver Interface:"
   entry $fL.receiverFrame.riFrame.riE \
         -textvariable receiver(interface,$ipVersion) -width 13

   pack $fL.receiverFrame.riFrame -fill x -expand true
   pack $fL.receiverFrame.riFrame.riL -padx 2 -pady 2 -side left
   pack $fL.receiverFrame.riFrame.riE -padx 2 -pady 2 -side right

   #user (login name) for remote commands execution
   frame $fL.receiverFrame.rlFrame
   label $fL.receiverFrame.rlFrame.rlL -text "Receiver Login:"
   entry $fL.receiverFrame.rlFrame.rlE \
         -textvariable receiver(login,$ipVersion) -width 13

   pack $fL.receiverFrame.rlFrame -fill x -expand true
   pack $fL.receiverFrame.rlFrame.rlL -side left -padx 2 -pady 2
   pack $fL.receiverFrame.rlFrame.rlE -padx 2 -pady 2 -side right

   #password:
   frame $fL.receiverFrame.rpFrame
   label $fL.receiverFrame.rpFrame.rpL -text "Receiver Password:"
   entry $fL.receiverFrame.rpFrame.rpE \
         -textvariable receiver(password,$ipVersion) -show "*" -width 13


   pack $fL.receiverFrame.rpFrame -fill x -expand true
   pack $fL.receiverFrame.rpFrame.rpL -padx 2 -pady 2 -side left
   pack $fL.receiverFrame.rpFrame.rpE -padx 2 -pady 2 -side right

   #checkbutton: local receiver?
   checkbutton $fL.receiverFrame.localReceiver -text "local" \
       -variable receiver(local,$ipVersion) -command LocalReceiver

   pack $fL.receiverFrame.localReceiver -pady 2

   #---------------------------------------------------------------------
   ##              multicast case for receiver
   #---------------------------------------------------------------------
   #  set a frame not just show v. Multicast keypressed
   #  killed frames: Interface - Login - Password - Local
   #---------------------------------------------------------------------
   #only receiver(address) has a meaning in multicast scenario
   #it reprezents the multicast address
   #multicast() variable is used for IP unicast addresses added to
   #multicast group

   frame $fL.mgroupFrame -bd 2 -relief groove
   label $fL.mgroupFrame.mgL -text "Group components"
   frame $fL.mgroupFrame.mgButtonsF
   button $fL.mgroupFrame.mgButtonsF.mgb1 \
        -text "Add Address\nto Group" -command AddGroupAddress
   button $fL.mgroupFrame.mgButtonsF.mgb2 \
        -text "Remove Selected\nAddress" -command RemoveGroupAddress

   #multicast scenario will enable this:
   pack $fL.mgroupFrame -fill both -expand true
   pack $fL.mgroupFrame.mgL $fL.mgroupFrame.mgButtonsF -fill x -expand true \
        -padx 2 -pady 2
   pack $fL.mgroupFrame.mgButtonsF.mgb1 \
        -side left -padx 2 -pady 2
   pack $fL.mgroupFrame.mgButtonsF.mgb2 \
        -side right -padx 5 -pady 2

   #the listbox with group-included addresses
   #one for ipv4, one for ipv6 case
   frame $fL.mgroupFrame.mgListF4
   set x(ipv4) [listbox $fL.mgroupFrame.mgListF4.mgListbox4 \
       -yscrollcommand "$fL.mgroupFrame.mgListF4.mgSY4 set" \
       -selectmode single]
   scrollbar $fL.mgroupFrame.mgListF4.mgSY4 -orient vertical \
       -command "$x(ipv4) yview"

   frame $fL.mgroupFrame.mgListF6
   set x(ipv6) [listbox $fL.mgroupFrame.mgListF6.mgListbox6 \
      -yscrollcommand "$fL.mgroupFrame.mgListF6.mgSY6 set" \
      -selectmode single]
   scrollbar $fL.mgroupFrame.mgListF6.mgSY6 -orient vertical \
      -command "$x(ipv6) yview"

   #pack only ipv4-related frame for now:
   pack $fL.mgroupFrame.mgListF4 -fill both -expand true -padx 2 -pady 2

   pack $x(ipv4) -side left -fill x -expand true
   pack $fL.mgroupFrame.mgListF4.mgSY4 -side right -fill y

   pack $x(ipv6) -side left -fill x -expand true
   pack $fL.mgroupFrame.mgListF6.mgSY6 -side right -fill y
   
   #Update multicast listbox data (if needed)
   foreach ipVersion {ipv4 ipv6} {
		for {set actual 1} {$actual <= $multicastEnd($ipVersion)} {incr actual} {
			$x($ipVersion) insert end \
				"$multicast(address,$ipVersion,$actual) : \
				$multicast(interface,$ipVersion,$actual)"
		}
		$x($ipVersion) see end
   }

   #---------------------------------------------------------------------
   #---------------------------------------------------------------------
   #    Right frame: images and host related widgets
   #---------------------------------------------------------------------
   #---------------------------------------------------------------------
   frame $fR.imgF -bd 2 -relief groove
   label $fR.imgF.unicastLabel -image uimage -anchor c
   label $fR.imgF.multicastLabel -image mimage -anchor c

   pack $fR.imgF
   pack $fR.imgF.unicastLabel -padx 3 -pady 3

   #management station (host) parameters:

   #a general frame
   frame $fR.hostF
   pack $fR.hostF -padx 2 -fill x

   #frame,label and entry for host address:

   frame $fR.hostF.aF -bd 2 -relief groove
   label $fR.hostF.aF.aL -text "Mgen Destination Address:"
   entry $fR.hostF.aF.aE -textvariable host(address,$ipVersion,$addressType)
   pack $fR.hostF.aF -fill x -side left
   pack $fR.hostF.aF.aL -side left -padx 2 -pady 2
   pack $fR.hostF.aF.aE -side right -padx 2 -pady 2

   #frame, label and entry for interface

   frame $fR.hostF.iF -bd 2 -relief groove
   label $fR.hostF.iF.iL -text "Interface"
   entry $fR.hostF.iF.iE -textvariable host(interface,$ipVersion,$addressType)

   pack $fR.hostF.iF -fill x -side right
   pack $fR.hostF.iF.iL -side left -padx 2 -pady 2
   pack $fR.hostF.iF.iE -side right -padx 2 -pady 2

   #results directory

   frame $fR.hostFd
   label $fR.hostFd.sL -text "Results Directory:   data /"
   entry $fR.hostFd.sE -textvariable scenarioDir($ipVersion,$addressType)
   pack  $fR.hostFd -fill x -pady 3
   pack  $fR.hostFd.sL -side left -padx 2 -pady 2
   pack  $fR.hostFd.sE -side right -padx 2 -pady 2 -fill x -expand true

   #######################
   ### CONFIG FILES Button
   #######################
   frame  $fR.conf    -bd 2 -relief groove
   pack   $fR.conf    -expand true -padx 2 -pady 2 

   label  $fR.conf.l -text "Configuration" 
   pack   $fR.conf.l -padx 3 -anchor c -side top -fill x   -padx 2 -pady 2

   frame  $fR.conf.e 
   pack   $fR.conf.e -pady 2 -fill x
   label  $fR.conf.e.l -text "Description:"
   #pack   $fR.conf.e.l -side left -padx 3
   #entry  $fR.conf.e.v -textvariable ConfInfo -width 45
   entry  $fR.conf.e.v -textvariable ConfInfo -width 50
   pack   $fR.conf.e.v -side right -padx 3


   label  $fR.conf.l1 -text "$GlobalFileConf" -relief raised -width 52
   pack   $fR.conf.l1 -padx 3 -anchor c -side top   -padx 2 -pady 2

   label  $fR.conf.l2 -text "Configuration files options:"
   #pack   $fR.conf.l2 -padx 3 -anchor c -side left           -padx 2 -pady 2

   frame  $fR.conf.b   
   pack   $fR.conf.b  -padx 2 -pady 2  -expand true

   button $fR.conf.b.save -text "Save"      -command { save_configuration_file $GlobalFileConf } -bd 2  -width 6
   button $fR.conf.b.load -text "Load"      -command load_as -bd 2  -width 6
   button $fR.conf.b.saveas -text "Save as" -command save_as -bd 2  -width 6
   button $fR.conf.b.quit -text "Quit" -command Quit -bd 2  -width 6
   button $fR.conf.b.os -text "OS"   -comman  OPTIONos -bd 2  -width 4
#   button $fR.conf.b.quit -text "XCALC" -command ProcessDLY2 -bd 2  -width 6
   pack $fR.conf.b.save $fR.conf.b.load $fR.conf.b.saveas $fR.conf.b.quit \
        $fR.conf.b.os -padx 2 -pady 2 -side left
}



###################################################################
#------------------------------------------------------------------
#            SO Option (External programs definitions)
#------------------------------------------------------------------
###################################################################
proc OPTIONos {} \
{
   global _xviewCMD
   global   _sshCMD

   #make a dialog window
   toplevel .opt
   wm title .opt "External programs Options"
   wm resizable .opt 0 0
   wm protocol .opt WM_DELETE_WINDOW {;#do nothing}

   frame .opt.centre 
   pack  .opt.centre -fill x -pady 3 -padx 3 -anchor n -expand true
   frame .opt.centre.right -bd 2 -relief groove
   pack  .opt.centre.right -fill x -pady 3 -padx 3 -anchor n -expand true -side top

   frame .opt.centre.right.xview
   pack  .opt.centre.right.xview -pady 3 -fill x
   label .opt.centre.right.xview.l -text "External X-Browser"
   pack  .opt.centre.right.xview.l -side left -padx 3
   entry .opt.centre.right.xview.e -textvariable _xviewCMD -width 30
   pack  .opt.centre.right.xview.e -side right -padx 3

   frame .opt.centre.right.ssh
   pack  .opt.centre.right.ssh -pady 3 -fill x
   label .opt.centre.right.ssh.l -text "SSH command"
   pack  .opt.centre.right.ssh.l -side left -padx 3
   entry .opt.centre.right.ssh.e -textvariable _sshCMD -width 30
   pack  .opt.centre.right.ssh.e -side right -padx 3

   button .opt.ok -text OK -command \
   {
      destroy .opt
   } -width 6
   pack .opt.ok
   grab .opt
   tkwait window .opt
}

proc ProcessDLY2 {} \
{
   global fPath
   global logDelayLB


   set newfile ""
   set newfile [tk_getOpenFile -defaultextension .dly \
   -initialdir data -filetypes {{{MGen-Broadband files} {.dly} {}}} ]
   if {$newfile == ""} {
   return
   }
    #if You don't want to launch the Test.... only process .DLY :(
   set fPath $newfile


      toplevel .title
      wm title .title "Making new MCL FILE"
      wm resizable .title 0 0
      wm protocol .title WM_DELETE_WINDOW {;#do nothing}

      frame .title.b
      pack  .title.b -side top
      frame .title.c
      pack  .title.c -side bottom -expand true
      label .title.b.l  -text "Do you want process the file to make the .MCL file?"
      pack  .title.b.l  -side top -padx 3 -fill x -expand true
      label .title.b.l1  -text "(you have to set the same configuration file of measure)"
      pack  .title.b.l1  -side top -padx 3 -fill x -expand true

      button .title.c.ok -text OK -command \
      {
         set done "1" 
         destroy .title
      } -width 6
      button .title.c.cancel -text Cancel -command \
      {
         set done "0"
         destroy .title
         return  
      } -width 6
      pack .title.c.ok .title.c.cancel  -side left -pady 2 
      ###################################################################
      #### Wait to END dialog
      ###################################################################
      grab .title
      tkwait window .title

   $logDelayLB delete 0 end
   ProcessData
}

###################################################################
#------------------------------------------------------------------
#                  M A I N - S E T U P  (other procedures)
#------------------------------------------------------------------
###################################################################

###procedures used in netmeter application
###names in defining order:
##RemoveGroupAddress
##AddgroupAddress
##Select
##LocalSender
##LocalReceiver

####################################################################
# ------------------------------------------------------------------
# REMOVEGROUPADDRESS (para el multicast chiamate da $fL.mgroupFrame)
# ------------------------------------------------------------------
####################################################################
###remove selected listbox address
###global variables used
proc RemoveGroupAddress {} \
{
   global x multicast multicastEnd exLine ipVersion

   #find the index of the selected line
   set exLine [$x($ipVersion) curselection]
   if ![string compare $exLine {}]  {
       return
   }
   #else:
   #delete the line from the listbox:
   $x($ipVersion) delete $exLine

   #now update the multicast array
   incr exLine
   while { $exLine < $multicastEnd($ipVersion)} {

   set multicast(local,$ipVersion,$exLine) $multicast(local,$ipVersion,[expr $exLine + 1])
   if !$multicast(local,$ipVersion,$exLine) {
   foreach i {address interface login password} {
       set multicast($i,$ipVersion,$exLine) $multicast($i,$ipVersion,[expr $exLine + 1])
       }
   }
   incr exLine
   }

   #erase last index values and decrement multicastEnd:
   foreach i {address interface login password} {
       set multicast($i,$ipVersion,$multicastEnd($ipVersion)) {}
       }

   incr multicastEnd($ipVersion) -1
}

##################################################################
# ----------------------------------------------------------------
# ADDGROUPADDRESS (para el multicast chiamate da $fL.mgroupFrame)
# ----------------------------------------------------------------
##################################################################
proc AddGroupAddress {} \
{
	###create a modal dialog to add data to
	###multicast array -addreses,interfaces etc.
	###indexed from 0 to $multicastEnd current value

	##needed variables
	global x multicast multicastEnd action address interface
	global login password local host ipVersion addressType
	set action not_yet
	set address {}
	set interface "eth0"
	set login {}
	set password {}
	set local 0

	#######################################################
	## define "internal" procedure Local
	## for checkbutton
	#######################################################
	proc Local {local} \
	{
		if $local {
			.dialog.aF.aE config -state disabled -fg gray
			.dialog.iF.iE config -state disabled -fg gray
			.dialog.lF.lE config -state disabled -fg gray
			.dialog.pF.pE config -state disabled -fg gray
		} else {
			.dialog.aF.aE config -state normal -fg black
			.dialog.iF.iE config -state normal -fg black
			.dialog.lF.lE config -state normal -fg black
			.dialog.pF.pE config -state normal -fg black
		}
	}
	#######################################################

	toplevel .dialog -bd 5
	wm title .dialog "Add address"
	wm resizable .dialog 0 0

	#frame, label, entry for address
	frame .dialog.aF
	label .dialog.aF.aL -text "New Address:"
	entry .dialog.aF.aE -textvariable address

	pack .dialog.aF -fill x
	pack .dialog.aF.aL -side left
	pack .dialog.aF.aE -side right

	#frame, label, entry for interface
	frame .dialog.iF
	label .dialog.iF.iL -text "Interface:"
	entry .dialog.iF.iE -textvariable interface -width 13
	pack .dialog.iF -fill x
	pack .dialog.iF.iL -side left
	pack .dialog.iF.iE -side right

	#frame, label, entry for login
	frame .dialog.lF
	label .dialog.lF.lL -text "Login:"
	entry .dialog.lF.lE -textvariable login -width 13
	pack  .dialog.lF -fill x
	pack  .dialog.lF.lL -side left
	pack  .dialog.lF.lE -side right

	#frame, label, entry for address
	frame .dialog.pF
	label .dialog.pF.pL -text "Password:"
	entry .dialog.pF.pE -textvariable password -width 13 -show "*"
	pack .dialog.pF -fill x
	pack .dialog.pF.pL -side left
	pack .dialog.pF.pE -side right

	#checkbutton for local option
	checkbutton .dialog.cB -variable local -text "local" \
	   -command { Local $local }
	pack .dialog.cB -pady 2

	#frame and ok&cancel buttons
	frame .dialog.bF -bd 5
	button .dialog.bF.okB -text "OK" -command {set action "ok"}
	button .dialog.bF.cancelB -text "Cancel" -command {set action "cancel"}
	pack .dialog.bF -fill x
	pack .dialog.bF.okB .dialog.bF.cancelB -side left -expand true

	#make dialog modal
	grab .dialog
	tkwait variable action
	grab release .dialog
	destroy .dialog

	if ![string compare $action "ok"] {
		if $local {
			# Fixed by Ren 27/08/02
			if ![info exists multicastEnd($ipVersion)] {
				set multicastEnd($ipVersion) 1
			} else {
				incr multicastEnd($ipVersion)
			}
			# --END-- Fixed by Ren 27/08/02
			set multicast(address,$ipVersion,$multicastEnd($ipVersion)) {}
			set multicast(interface,$ipVersion,$multicastEnd($ipVersion)) {}
			set multicast(login,$ipVersion,$multicastEnd($ipVersion)) {}
			set multicast(password,$ipVersion,$multicastEnd($ipVersion)) {}
			set multicast(local,$ipVersion,$multicastEnd($ipVersion)) $local ;#1 the only thing that counts
			#update the listbox data:
			$x($ipVersion) insert end "local"
			$x($ipVersion) see end
		} elseif [string compare [string trim $address] {}] {
			# Fixed by Ren 27/08/02
			if ![info exists multicastEnd($ipVersion)] {
				set multicastEnd($ipVersion) 1
			} else {
				incr multicastEnd($ipVersion)
			}
			# --END-- Fixed by Ren 27/08/02
			set multicast(address,$ipVersion,$multicastEnd($ipVersion)) $address
			set multicast(interface,$ipVersion,$multicastEnd($ipVersion)) $interface
			set multicast(login,$ipVersion,$multicastEnd($ipVersion)) $login
			set multicast(password,$ipVersion,$multicastEnd($ipVersion)) $password
			set multicast(local,$ipVersion,$multicastEnd($ipVersion)) $local ;#0
			#update the listbox data:
			$x($ipVersion) insert end \
				"$multicast(address,$ipVersion,$multicastEnd($ipVersion)) : \
				$multicast(interface,$ipVersion,$multicastEnd($ipVersion))"
			$x($ipVersion) see end
		}
	}
}

################################################################
# --------------------------------------------------------------
#			SELECT
# --------------------------------------------------------------
################################################################
proc Select {} \
{
   ###select visible frames, according with selected scenario
   ###(configuration of checked radiobuttons)
   ##necesary variables declarations
   global ipVersion addressType newState fL fR u4tF uni4 u6tF uni6
   global m4tF mul4 m6tF mul6 x sender receiver host multicastEnd

   ################################################################ 
   # aggiorna le variabili del vettore sender in funzione dei valori
   # dei Radio_button   "ipversion"    y    "addressType" con i valori
   # delle entry  fL.senderFrame.saframe.ENTRY
   ################################################################

	###update entries to store data in the right variables:
	##PAGE ONE
	##sender:
	$fL.senderFrame.saFrame.saE config \
	    -textvariable sender(address,$ipVersion,$addressType)
	$fL.senderFrame.siFrame.siE config \
	    -textvariable sender(interface,$ipVersion,$addressType)
	$fL.senderFrame.slFrame.slE config \
	    -textvariable sender(login,$ipVersion,$addressType)
	$fL.senderFrame.spFrame.spE config \
	    -textvariable sender(password,$ipVersion,$addressType)
	$fL.senderFrame.localSender config \
	    -variable sender(local,$ipVersion,$addressType)

	##receiver:
	$fL.receiverFrame.raFrame.raE config \
	    -textvariable receiver(address,$ipVersion,$addressType)
	$fL.receiverFrame.riFrame.riE config \
	    -textvariable receiver(interface,$ipVersion)
	$fL.receiverFrame.rlFrame.rlE config \
	    -textvariable receiver(login,$ipVersion)
	$fL.receiverFrame.rpFrame.rpE config \
	    -textvariable receiver(password,$ipVersion)
	$fL.receiverFrame.localReceiver config \
	    -variable receiver(local,$ipVersion)

	#the multicast addresses are treated in AddGroupAddress
	#and RemoveGroupAddress

	#host and scenarioDir
	$fR.hostF.aF.aE config \
	    -textvariable host(address,$ipVersion,$addressType)
	$fR.hostF.iF.iE config \
	    -textvariable host(interface,$ipVersion,$addressType)
	$fR.hostFd.sE config \
	    -textvariable scenarioDir($ipVersion,$addressType)

   ###############################################################
   ###############################################################
   ##############################################################
	#make a call to LocalSender and LocalReceiver
	#to fix the correct state of all entries
	LocalSender
	LocalReceiver


	###find the current scenario type:
	if ![string compare $ipVersion "ipv4"] {
		if { $addressType == "unicast"} {
			set newState u4
		} else {
			set newState m4
		}
	} else {
		if { $addressType == "unicast"} {
			set newState u6
		} else {
			set newState m6
		}

	}
	#depending on the scenario, show/hide widgets

	switch $newState {
	u4
	{
		##unicast -ipv4
		#page 1:

		#disable:
		pack forget $fL.mgroupFrame           #  multicast case for receiver
		pack forget $fR.imgF.multicastLabel
		#enable:                              # il settaggio di "indirizzi-local-passw"
		pack $fL.receiverFrame.riFrame -fill x -expand true
		pack $fL.receiverFrame.rlFrame -fill x -expand true
		pack $fL.receiverFrame.rpFrame -fill x -expand true
		pack $fL.receiverFrame.localReceiver -pady 2
		pack $fR.imgF.unicastLabel -padx 3 -pady 3

		##page 2:
		#disable:
		pack forget $u6tF
		pack forget $m4tF
		pack forget $m6tF

		#enable:
		pack $u4tF -fill both -expand true


		#if I want do enable/disable this page at unicast/multicast
		#switching, I must change color of page title:
		#.c itemconfigure $page(2,texto) -fill black
		#.c itemconfigure $page(3,texto) -fill black

		##page 3:
		#disable:
		pack forget $uni6
		pack forget $mul4
		pack forget $mul6

		#enable:
		pack $uni4 -fill both -expand true
	}
	u6
		{
		##unicast -ipv6
		#page 1:

		#disable:
		pack forget $fL.mgroupFrame
		pack forget $fR.imgF.multicastLabel
		#enable:
		pack $fL.receiverFrame.riFrame -fill x -expand true
		pack $fL.receiverFrame.rlFrame -fill x -expand true
		pack $fL.receiverFrame.rpFrame -fill x -expand true
		pack $fL.receiverFrame.localReceiver -pady 2
		pack $fR.imgF.unicastLabel -padx 3 -pady 3

		#page 2:
		#disable:
		pack forget $u4tF
		pack forget $m4tF
		pack forget $m6tF

		#enable:
		pack $u6tF -fill both -expand true

		#if I want do enable/disable this page at unicast/multicast
		#switching, I must change color of page title:
		#.c itemconfigure $page(2,texto) -fill black
		#.c itemconfigure $page(3,texto) -fill black

		#page 3:
		#disable:
		pack forget $uni4
		pack forget $mul4
		pack forget $mul6

		#enable:
		pack $uni6 -fill both -expand true
	}
	m4
	{
		##multicast-ipv4
		#page 1

		#disable
		pack forget $fL.receiverFrame.riFrame
		pack forget $fL.receiverFrame.rlFrame
		pack forget $fL.receiverFrame.rpFrame
		pack forget $fL.receiverFrame.localReceiver
		pack forget $fL.mgroupFrame.mgListF6
		pack forget $fR.imgF.unicastLabel

		#enable
		pack $fL.mgroupFrame
		pack $fL.mgroupFrame.mgListF4 -fill both -expand true -padx 2 -pady 2
		pack $fR.imgF.multicastLabel -padx 3 -pady 3

		#page 2:
		#disable:
		pack forget $u4tF
		pack forget $u6tF
		pack forget $m6tF

		#enable:
		pack $m4tF -fill both -expand true

		#if I want do enable/disable this page at unicast/multicast
		#switching, I must change color of page title:
		#    .c itemconfigure $page(2,texto) -fill gray
		#    .c itemconfigure $page(3,texto) -fill gray

		#page 3:
		#disable:
		pack forget $uni4
		pack forget $uni6
		pack forget $mul6

		#enable:
		pack $mul4 -fill both -expand true
	}
	m6
	{
		##multicast-ipv6
		#page 1

		#disable
		pack forget $fL.receiverFrame.riFrame
		pack forget $fL.receiverFrame.rlFrame
		pack forget $fL.receiverFrame.rpFrame
		pack forget $fL.receiverFrame.localReceiver
		pack forget $fL.mgroupFrame.mgListF4
		pack forget $fR.imgF.unicastLabel
		#tk_messageBox -message "Sorry! the Multicast scenario is not available!"

		#enable
		pack $fL.mgroupFrame
		pack $fL.mgroupFrame.mgListF6 -fill both -expand true -padx 2 -pady 2
		pack $fR.imgF.multicastLabel -padx 3 -pady 3

		#page 2:
		#disable:
		pack forget $u4tF
		pack forget $u6tF
		pack forget $m4tF

		#enable:
		pack $m6tF -fill both -expand true


		#if I want do enable/disable this page at unicast/multicast
		#switching, I must change color of page title:
		#    .c itemconfigure $page(2,texto) -fill gray
		#    .c itemconfigure $page(3,texto) -fill gray

		#page 3:
		#disable:
		pack forget $uni4
		pack forget $uni6
		pack forget $mul4

		#enable:
		pack $mul6 -fill both -expand true
	}
	}
}




################################################################
# --------------------------------------------------------------
#		LOCALSENDER  (interna di Select)
# ---------------------------------------------------------------
################################################################
#### disable/enable sender widgets
proc LocalSender {}  \
{

    global fL sender ipVersion addressType

    if $sender(local,$ipVersion,$addressType) {
       $fL.senderFrame.saFrame.saE config -state disabled -fg gray
       $fL.senderFrame.siFrame.siE config -state disabled -fg gray
       $fL.senderFrame.slFrame.slE config -state disabled -fg gray
       $fL.senderFrame.spFrame.spE config -state disabled -fg gray

    } else {

       $fL.senderFrame.saFrame.saE config -state normal -fg black
       $fL.senderFrame.siFrame.siE config -state normal -fg black
       $fL.senderFrame.slFrame.slE config -state normal -fg black
       $fL.senderFrame.spFrame.spE config -state normal -fg black
    }
 }

################################################################
# --------------------------------------------------------------
#		LOCALRECEIVER  (interna di Select)
# ---------------------------------------------------------------
################################################################
#### disable/enable receiver widgets
proc LocalReceiver {} \
{

    global fL receiver ipVersion

    if $receiver(local,$ipVersion) {       #button checked

       $fL.receiverFrame.raFrame.raE config -state disabled -fg gray
       $fL.receiverFrame.riFrame.riE config -state disabled -fg gray
       $fL.receiverFrame.rlFrame.rlE config -state disabled -fg gray
       $fL.receiverFrame.rpFrame.rpE config -state disabled -fg gray

    } else {

       $fL.receiverFrame.raFrame.raE config -state normal -fg black
       $fL.receiverFrame.riFrame.riE config -state normal -fg black
       $fL.receiverFrame.rlFrame.rlE config -state normal -fg black
       $fL.receiverFrame.rpFrame.rpE config -state normal -fg black
    }
}

################################################################
# --------------------------------------------------------------
#		QUIT  (not DONE)
# ---------------------------------------------------------------
################################################################
#### MAY BE :save data to config. file and then exit
proc Quit {}  \
{
    exit
}



################################################################
# --------------------------------------------------------------
#	Various Misc Options to avoid bugs
# ---------------------------------------------------------------
################################################################
#
proc verifyRemotePath mgenArray \
{
	upvar $mgenArray mgen
	
	global debug
	
	if $debug {
		puts stderr "verifyRemotePath: BEGIN"
		puts stderr "                       Login:    $mgen(login)"
		puts stderr "                       HostName: $mgen(host)"
		puts stderr "                       Path:     $mgen(path)"
	}
	
	set textOut ""
	if {[catch {eval exec {ssh -x $mgen(login)@$mgen(host) /usr/bin/which $mgen(path) 2> /tmp/netmeterErr }} ] == 0} {
		set textOut "$mgen(path) in $mgen(host) found, great!"
		set errorFound 0
	} else {
		set textOut "$mgen(path) in $mgen(host) not found"
		set errorFound 1
	}
	if $debug {
		puts stderr "                       $textOut"
		puts stderr "verifyRemotePath: END"
	}
	if $errorFound {
		tk_messageBox -message "Attention remote failure:\n$textOut!!!!!"
		update idletasks
	}
	return $errorFound
}

proc verifyLocalPath { l_Path verbose } \
{
	upvar $l_Path path
	
	global debug
	
	if $debug {
		puts stderr "verifyLocalPath: BEGIN"
		puts stderr "                      Path:     $l_Path"
	}
	
	set textOut ""
	if {[catch {eval exec { which $l_Path }} ] == 0} {
		set textOut "$l_Path found, great!"
		set errorFound 0
	} else {
		set textOut "Path not found, can't continue."
		set errorFound 1
	}
	if $debug {
		puts stderr "                       $textOut"
		puts stderr "verifyRemotePath: END"
	}
	if { $errorFound && $verbose } {
		tk_messageBox -message "Attention $l_Path failure:\n$textOut!!!!!"
		update idletasks
	}
	return $errorFound
}

proc verifyVersion f_Exec \
{
	set version [exec bin/verify.sh $f_Exec]
	return version
}

proc verifyMcalc f_Exec \
{
	set version [exec $f_Exec -v]
	if { [ string compare $version "MCALC Version 3.3" ] && [ string compare $version "MCALC Version 3.1 IPv6" ] } {
		return 0
	} else {
		return 1
	}
}

proc calcFlow {file_mine flow_mine} {
	global debug

	set ret_lines 0

	if $debug {
		puts stderr "calcFlow: BEGIN"
		puts stderr "               File to count: $file_mine"
		puts stderr "               Flow to count: $flow_mine"
	}

	if [catch {open $file_mine} f_ID] {
		set ret_lines $zoom_size	;# Not fool around, only put default value if file doesn't exist
		return
	}
	while {![eof $f_ID]} {
		gets $f_ID line
		set isFlow [string first $flow_mine $line]
		if {$isFlow >= 0} {
			incr ret_lines
		}
	}
	if $debug {
		puts stderr "               Returned Packets: $ret_lines"
		puts stderr "calcFlow: END"
	}
	return $ret_lines
}

proc giveBandwidth {protocol rate size bandw} {
	upvar $bandw bw
        if { [ regexp {(^[0-9]+$)} $rate ] && [ regexp {(^[0-9]+$)} $size ] && $rate > 0 && $size > 0} {
		if { $protocol == "ipv4" } {
			set bw(atm) [ exec bin/bw_rate -k -R $rate $size ]
			set bw(eth) [ exec bin/bw_rate_eth -k -R $rate $size ]
		} else {
			set bw(atm) [ exec bin/bw_rate -k -6 -R $rate $size ]
			set bw(eth) [ exec bin/bw_rate_eth -k -6 -R $rate $size ]
		}
	} else {
		set bw(atm) "N/A"
		set bw(eth) "N/A"
	}
}

proc randazAZchar {} {
     return [format %c [expr {int(rand() * 26) + [expr {int(rand() * 10) > 5 ? 97 : 65}]}]]
}
proc randazAZstring {length} {
     set s {}
     for {set i $length} {$i > 0} {incr i -1} {append s [randazAZchar]}
     return $s
}
proc makeTempFile {template} {
     if {[set xi [string first "X" $template]] == -1} {
         set n $template
         set p {}
         set rl 0
     } else {
         set n [string range $template 0 [expr {$xi - 1}]]
         set p [pid]
         set rl [expr {[string length $template] - $xi - [string length $p]}]
         if {$rl < 0} {
             set p [string range $p 0 end-[expr {$rl * -1}]]
             set rl 0
         }
     }
     for {set i [expr {int(pow(26, ($rl <= 6 ? $rl : 6)))}]} {$i > 0} {incr i -1} {
         set fn $n[randazAZstring $rl]$p
         if {![catch {open $fn {CREAT EXCL RDWR} 0600} fd]} {
             file delete $fn
             return $fd
         }
         if {[lindex $::errorCode 1] ne "EEXIST" || [lindex $::errorCode 0] ne "POSIX"} {
             break
         }
     }
     return -code error -errorcode $::errorCode $fd
}